=== modified file 'NEWS'
--- NEWS	2007-10-30 14:17:45 +0000
+++ NEWS	2007-11-07 10:08:49 +0000
@@ -1,11 +1,21 @@
-0.92.0	UNRELEASED
+0.93.0  UNRELEASED
+
+
+
+0.92.0	2007-11-07
 
  UI
 
   * Removed email part of committer id from committer column in the revision
     history window. (Daniel Schierbeck)
+    
   * Removed date column from revision history window. (Daniel Schierbeck)
+  
   * Rearranged the logview into a notebook. (Daniel Schierbeck)
+  
+  * Redesigned the Commit dialog. (John Arbash Meinel)
+  
+  * Broken lines support made optional. (Daniel Schierbeck)
 
  FEATURES
  

=== modified file '__init__.py'
--- __init__.py	2007-11-06 18:02:09 +0000
+++ __init__.py	2007-11-07 14:19:09 +0000
@@ -32,9 +32,11 @@
 visualise         Graphically visualise this branch. 
 """
 
+import sys
+
 import bzrlib
 
-version_info = (0, 92, 0, 'dev', 0)
+version_info = (0, 93, 0, 'dev', 0)
 
 if version_info[3] == 'final':
     version_string = '%d.%d.%d' % version_info[:3]
@@ -332,7 +334,7 @@
     """GTK+ commit dialog
 
     Graphical user interface for committing revisions"""
-    
+
     aliases = [ "gci" ]
     takes_args = []
     takes_options = []
@@ -351,12 +353,21 @@
             (wt, path) = workingtree.WorkingTree.open_containing(filename)
             br = wt.branch
         except NoWorkingTree, e:
-            path = e.base
-            (br, path) = branch.Branch.open_containing(path)
-
-        commit = CommitDialog(wt, path, not br)
-        commit.run()
-
+            from dialog import error_dialog
+            error_dialog(_('Directory does not have a working tree'),
+                         _('Operation aborted.'))
+            return 1 # should this be retval=3?
+
+        # It is a good habit to keep things locked for the duration, but it
+        # could cause difficulties if someone wants to do things in another
+        # window... We could lock_read() until we actually go to commit
+        # changes... Just a thought.
+        wt.lock_write()
+        try:
+            dlg = CommitDialog(wt)
+            return dlg.run()
+        finally:
+            wt.unlock()
 
 
 class cmd_gstatus(GTKCommand):
@@ -608,6 +619,68 @@
 register_command(cmd_gselftest)
 
 
+class cmd_test_gtk(GTKCommand):
+    """Version of selftest that just runs the gtk test suite."""
+
+    takes_options = ['verbose',
+                     Option('one', short_name='1',
+                            help='stop when one test fails'),
+                     Option('benchmark', help='run the benchmarks.'),
+                     Option('lsprof-timed',
+                     help='generate lsprof output for benchmarked'
+                          ' sections of code.'),
+                     Option('list-only',
+                     help='list the tests instead of running them'),
+                     Option('randomize', type=str, argname="SEED",
+                     help='randomize the order of tests using the given'
+                          ' seed or "now" for the current time'),
+                    ]
+    takes_args = ['testspecs*']
+
+    def run(self, verbose=None, one=False, benchmark=None,
+            lsprof_timed=None, list_only=False, randomize=None,
+            testspecs_list=None):
+        from bzrlib import __path__ as bzrlib_path
+        from bzrlib.tests import selftest
+
+        print '%10s: %s' % ('bzrlib', bzrlib_path[0])
+        if benchmark:
+            print 'No benchmarks yet'
+            return 3
+
+            test_suite_factory = bench_suite
+            if verbose is None:
+                verbose = True
+            # TODO: should possibly lock the history file...
+            benchfile = open(".perf_history", "at", buffering=1)
+        else:
+            test_suite_factory = test_suite
+            if verbose is None:
+                verbose = False
+            benchfile = None
+
+        if testspecs_list is not None:
+            pattern = '|'.join(testspecs_list)
+        else:
+            pattern = ".*"
+
+        try:
+            result = selftest(verbose=verbose,
+                              pattern=pattern,
+                              stop_on_failure=one,
+                              test_suite_factory=test_suite_factory,
+                              lsprof_timed=lsprof_timed,
+                              bench_history=benchfile,
+                              list_only=list_only,
+                              random_seed=randomize,
+                             )
+        finally:
+            if benchfile is not None:
+                benchfile.close()
+
+register_command(cmd_test_gtk)
+
+
 import gettext
 gettext.install('olive-gtk')
 

=== modified file 'annotate/gannotate.py'
--- annotate/gannotate.py	2007-11-02 14:49:57 +0000
+++ annotate/gannotate.py	2007-11-03 16:27:35 +0000
@@ -64,6 +64,7 @@
         self.branch = branch
         self.tree = tree
         self.file_id = file_id
+        self.logview.set_file_id(file_id)
         self.revision_id = getattr(tree, 'get_revision_id', 
                                    lambda: CURRENT_REVISION)()
         

=== modified file 'commit.py'
--- commit.py	2007-10-22 18:37:19 +0000
+++ commit.py	2007-11-02 22:08:54 +0000
@@ -25,9 +25,11 @@
 import pango
 
 import os.path
+import re
 
 from bzrlib import errors, osutils
 from bzrlib.trace import mutter
+from bzrlib.util import bencode
 
 from dialog import error_dialog, question_dialog
 from errors import show_bzr_error
@@ -39,381 +41,685 @@
 except ImportError:
     have_dbus = False
 
+
+def pending_revisions(wt):
+    """Return a list of pending merges or None if there are none of them.
+
+    Arguably this should be a core function, and
+    ``bzrlib.status.show_pending_merges`` should be built on top of it.
+
+    :return: [(rev, [children])]
+    """
+    parents = wt.get_parent_ids()
+    if len(parents) < 2:
+        return None
+
+    # The basic pending merge algorithm uses the same algorithm as
+    # bzrlib.status.show_pending_merges
+    pending = parents[1:]
+    branch = wt.branch
+    last_revision = parents[0]
+
+    if last_revision is not None:
+        try:
+            ignore = set(branch.repository.get_ancestry(last_revision,
+                                                        topo_sorted=False))
+        except errors.NoSuchRevision:
+            # the last revision is a ghost : assume everything is new
+            # except for it
+            ignore = set([None, last_revision])
+    else:
+        ignore = set([None])
+
+    pm = []
+    for merge in pending:
+        ignore.add(merge)
+        try:
+            rev = branch.repository.get_revision(merge)
+            children = []
+            pm.append((rev, children))
+
+            # This does need to be topo sorted, so we search backwards
+            inner_merges = branch.repository.get_ancestry(merge)
+            assert inner_merges[0] is None
+            inner_merges.pop(0)
+            for mmerge in reversed(inner_merges):
+                if mmerge in ignore:
+                    continue
+                rev = branch.repository.get_revision(mmerge)
+                children.append(rev)
+
+                ignore.add(mmerge)
+        except errors.NoSuchRevision:
+            print "DEBUG: NoSuchRevision:", merge
+
+    return pm
+
+
 class CommitDialog(gtk.Dialog):
-    """ New implementation of the Commit dialog. """
-    def __init__(self, wt, wtpath, notbranch, selected=None, parent=None):
-        """ Initialize the Commit Dialog. """
+    """Implementation of Commit."""
+
+    def __init__(self, wt, selected=None, parent=None):
         gtk.Dialog.__init__(self, title="Commit - Olive",
                                   parent=parent,
                                   flags=0,
                                   buttons=(gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL))
-        
-        # Get arguments
-        self.wt = wt
-        self.wtpath = wtpath
-        self.notbranch = notbranch
-        self.selected = selected
-        
-        # Set the delta
-        self.old_tree = self.wt.branch.repository.revision_tree(self.wt.branch.last_revision())
-        self.delta = self.wt.changes_from(self.old_tree)
-        
-        # Get pending merges
-        self.pending = self._pending_merges(self.wt)
-        
-        # Do some preliminary checks
-        self._is_checkout = False
-        self._is_pending = False
-        if self.wt is None and not self.notbranch:
-            error_dialog(_('Directory does not have a working tree'),
-                         _('Operation aborted.'))
-            self.close()
-            return
-
-        if self.notbranch:
-            error_dialog(_('Directory is not a branch'),
-                         _('You can perform this action only in a branch.'))
-            self.close()
-            return
+        self._question_dialog = question_dialog
+
+        self._wt = wt
+        # TODO: Do something with this value, it is used by Olive
+        #       It used to set all changes but this one to False
+        self._selected = selected
+        self._enable_per_file_commits = True
+        self._commit_all_changes = True
+        self.committed_revision_id = None # Nothing has been committed yet
+
+        self.setup_params()
+        self.construct()
+        self.fill_in_data()
+
+    def setup_params(self):
+        """Setup the member variables for state."""
+        self._basis_tree = self._wt.basis_tree()
+        self._delta = None
+        self._pending = pending_revisions(self._wt)
+
+        self._is_checkout = (self._wt.branch.get_bound_location() is not None)
+
+    def fill_in_data(self):
+        # Now that we are built, handle changes to the view based on the state
+        self._fill_in_pending()
+        self._fill_in_diff()
+        self._fill_in_files()
+        self._fill_in_checkout()
+        self._fill_in_per_file_info()
+
+    def _fill_in_pending(self):
+        if not self._pending:
+            self._pending_box.hide()
+            return
+
+        # TODO: We'd really prefer this to be a nested list
+        for rev, children in self._pending:
+            rev_info = self._rev_to_pending_info(rev)
+            self._pending_store.append([
+                rev_info['revision_id'],
+                rev_info['date'],
+                rev_info['committer'],
+                rev_info['summary'],
+                ])
+            for child in children:
+                rev_info = self._rev_to_pending_info(child)
+                self._pending_store.append([
+                    rev_info['revision_id'],
+                    rev_info['date'],
+                    rev_info['committer'],
+                    rev_info['summary'],
+                    ])
+        self._pending_box.show()
+
+    def _fill_in_files(self):
+        # We should really use add a progress bar of some kind.
+        # While we fill in the view, hide the store
+        store = self._files_store
+        self._treeview_files.set_model(None)
+
+        added = _('added')
+        removed = _('removed')
+        renamed = _('renamed')
+        renamed_and_modified = _('renamed and modified')
+        modified = _('modified')
+        kind_changed = _('kind changed')
+
+        # The store holds:
+        # [file_id, real path, checkbox, display path, changes type, message]
+        # _iter_changes returns:
+        # (file_id, (path_in_source, path_in_target),
+        #  changed_content, versioned, parent, name, kind,
+        #  executable)
+
+        all_enabled = (self._selected is None)
+        # The first entry is always the 'whole tree'
+        all_iter = store.append([None, None, all_enabled, 'All Files', '', ''])
+        initial_cursor = store.get_path(all_iter)
+        # should we pass specific_files?
+        self._wt.lock_read()
+        self._basis_tree.lock_read()
+        try:
+            from diff import _iter_changes_to_status
+            for (file_id, real_path, change_type, display_path
+                ) in _iter_changes_to_status(self._basis_tree, self._wt):
+                if self._selected and real_path != self._selected:
+                    enabled = False
+                else:
+                    enabled = True
+                item_iter = store.append([
+                    file_id,
+                    real_path.encode('UTF-8'),
+                    enabled,
+                    display_path.encode('UTF-8'),
+                    change_type,
+                    '', # Initial comment
+                    ])
+                if self._selected and enabled:
+                    initial_cursor = store.get_path(item_iter)
+        finally:
+            self._basis_tree.unlock()
+            self._wt.unlock()
+
+        self._treeview_files.set_model(store)
+        self._last_selected_file = None
+        # This sets the cursor, which causes the expander to close, which
+        # causes the _file_message_text_view to never get realized. So we have
+        # to give it a little kick, or it warns when we try to grab the focus
+        self._treeview_files.set_cursor(initial_cursor)
+
+        def _realize_file_message_tree_view(*args):
+            self._file_message_text_view.realize()
+        self.connect_after('realize', _realize_file_message_tree_view)
+
+    def _fill_in_diff(self):
+        self._diff_view.set_trees(self._wt, self._basis_tree)
+
+    def _fill_in_checkout(self):
+        if not self._is_checkout:
+            self._check_local.hide()
+            return
+        if have_dbus:
+            bus = dbus.SystemBus()
+            proxy_obj = bus.get_object('org.freedesktop.NetworkManager',
+                                       '/org/freedesktop/NetworkManager')
+            dbus_iface = dbus.Interface(proxy_obj,
+                                        'org.freedesktop.NetworkManager')
+            try:
+                # 3 is the enum value for STATE_CONNECTED
+                self._check_local.set_active(dbus_iface.state() != 3)
+            except dbus.DBusException, e:
+                # Silently drop errors. While DBus may be
+                # available, NetworkManager doesn't necessarily have to be
+                mutter("unable to get networkmanager state: %r" % e)
+        self._check_local.show()
+
+    def _fill_in_per_file_info(self):
+        config = self._wt.branch.get_config()
+        enable_per_file_commits = config.get_user_option('per_file_commits')
+        if (enable_per_file_commits is None
+            or enable_per_file_commits.lower()
+                not in ('y', 'yes', 'on', 'enable', '1', 't', 'true')):
+            self._enable_per_file_commits = False
         else:
-            if self.wt.branch.get_bound_location() is not None:
-                # we have a checkout, so the local commit checkbox must appear
-                self._is_checkout = True
-            
-            if self.pending:
-                # There are pending merges, file selection not supported
-                self._is_pending = True
-        
-        # Create the widgets
+            self._enable_per_file_commits = True
+        if not self._enable_per_file_commits:
+            self._file_message_expander.hide()
+            self._global_message_label.set_markup(_('<b>Commit Message</b>'))
+
+    def _compute_delta(self):
+        self._delta = self._wt.changes_from(self._basis_tree)
+
+    def construct(self):
+        """Build up the dialog widgets."""
+        # The primary pane which splits it into left and right (adjustable)
+        # sections.
+        self._hpane = gtk.HPaned()
+
+        self._construct_left_pane()
+        self._construct_right_pane()
+        self._construct_action_pane()
+
+        self.vbox.pack_start(self._hpane)
+        self._hpane.show()
+        self.set_focus(self._global_message_text_view)
+
+        self._construct_accelerators()
+        self._set_sizes()
+
+    def _set_sizes(self):
+        # This seems like a reasonable default, we might like it to
+        # be a bit wider, so that by default we can fit an 80-line diff in the
+        # diff window.
+        # Alternatively, we should be saving the last position/size rather than
+        # setting it to a fixed value every time we start up.
+        screen = self.get_screen()
+        monitor = 0 # We would like it to be the monitor we are going to
+                    # display on, but I don't know how to figure that out
+                    # Only really useful for freaks like me that run dual
+                    # monitor, with different sizes on the monitors
+        monitor_rect = screen.get_monitor_geometry(monitor)
+        width = int(monitor_rect.width * 0.66)
+        height = int(monitor_rect.height * 0.66)
+        self.set_default_size(width, height)
+        self._hpane.set_position(300)
+
+    def _construct_accelerators(self):
+        group = gtk.AccelGroup()
+        group.connect_group(gtk.gdk.keyval_from_name('N'),
+                            gtk.gdk.CONTROL_MASK, 0, self._on_accel_next)
+        self.add_accel_group(group)
+
+    def _construct_left_pane(self):
+        self._left_pane_box = gtk.VBox(homogeneous=False, spacing=5)
+        self._construct_file_list()
+        self._construct_pending_list()
+
+        self._check_local = gtk.CheckButton(_("_Only commit locally"),
+                                            use_underline=True)
+        self._left_pane_box.pack_end(self._check_local, False, False)
+        self._check_local.set_active(False)
+
+        self._hpane.pack1(self._left_pane_box, resize=False, shrink=False)
+        self._left_pane_box.show()
+
+    def _construct_right_pane(self):
+        # TODO: I really want to make it so the diff view gets more space than
+        # the global commit message, and the per-file commit message gets even
+        # less. When I did it with wxGlade, I set it to 4 for diff, 2 for
+        # commit, and 1 for file commit, and it looked good. But I don't seem
+        # to have a way to do that with the gtk boxes... :( (Which is extra
+        # weird since wx uses gtk on Linux...)
+        self._right_pane_table = gtk.Table(rows=10, columns=1, homogeneous=False)
+        self._right_pane_table.set_row_spacings(5)
+        self._right_pane_table.set_col_spacings(5)
+        self._right_pane_table_row = 0
+        self._construct_diff_view()
+        self._construct_file_message()
+        self._construct_global_message()
+
+        self._right_pane_table.show()
+        self._hpane.pack2(self._right_pane_table, resize=True, shrink=True)
+
+    def _construct_action_pane(self):
         self._button_commit = gtk.Button(_("Comm_it"), use_underline=True)
-        self._expander_files = gtk.Expander(_("File(s) to commit"))
-        self._vpaned_main = gtk.VPaned()
-        self._scrolledwindow_files = gtk.ScrolledWindow()
-        self._scrolledwindow_message = gtk.ScrolledWindow()
-        self._treeview_files = gtk.TreeView()
-        self._vbox_message = gtk.VBox()
-        self._label_message = gtk.Label(_("Commit message:"))
-        self._textview_message = gtk.TextView()
-        
-        if self._is_pending:
-            self._expander_merges = gtk.Expander(_("Pending merges"))
-            self._vpaned_list = gtk.VPaned()
-            self._scrolledwindow_merges = gtk.ScrolledWindow()
-            self._treeview_merges = gtk.TreeView()
-            self._scrolledwindow_merges.set_policy(gtk.POLICY_AUTOMATIC,
-                                                   gtk.POLICY_AUTOMATIC)
-
-        # Set callbacks
         self._button_commit.connect('clicked', self._on_commit_clicked)
-        self._treeview_files.connect('row_activated', self._on_treeview_files_row_activated)
-        
-        # Set properties
-        self._scrolledwindow_files.set_policy(gtk.POLICY_AUTOMATIC,
-                                              gtk.POLICY_AUTOMATIC)
-        self._scrolledwindow_message.set_policy(gtk.POLICY_AUTOMATIC,
-                                                gtk.POLICY_AUTOMATIC)
-        self._textview_message.modify_font(pango.FontDescription("Monospace"))
-        self.set_default_size(500, 500)
-        self._vpaned_main.set_position(200)
         self._button_commit.set_flags(gtk.CAN_DEFAULT)
-
-        # Construct the dialog
+        self._button_commit.show()
         self.action_area.pack_end(self._button_commit)
-        
-        self._scrolledwindow_files.add(self._treeview_files)
-        self._scrolledwindow_message.add(self._textview_message)
-        
-        self._expander_files.add(self._scrolledwindow_files)
-        
-        self._vbox_message.pack_start(self._label_message, False, False)
-        self._vbox_message.pack_start(self._scrolledwindow_message, True, True)
-        
-        if self._is_pending:        
-            self._expander_merges.add(self._scrolledwindow_merges)
-            self._scrolledwindow_merges.add(self._treeview_merges)
-            self._vpaned_list.add1(self._expander_files)
-            self._vpaned_list.add2(self._expander_merges)
-            self._vpaned_main.add1(self._vpaned_list)
-        else:
-            self._vpaned_main.add1(self._expander_files)
-
-        self._vpaned_main.add2(self._vbox_message)
-        
-        self.vbox.pack_start(self._vpaned_main, True, True)
-        if self._is_checkout: 
-            self._check_local = gtk.CheckButton(_("_Only commit locally"),
-                                                use_underline=True)
-            self.vbox.pack_start(self._check_local, False, False)
-            if have_dbus:
-                bus = dbus.SystemBus()
-                proxy_obj = bus.get_object('org.freedesktop.NetworkManager', 
-                              '/org/freedesktop/NetworkManager')
-                dbus_iface = dbus.Interface(
-                        proxy_obj, 'org.freedesktop.NetworkManager')
-                try:
-                    # 3 is the enum value for STATE_CONNECTED
-                    self._check_local.set_active(dbus_iface.state() != 3)
-                except dbus.DBusException, e:
-                    # Silently drop errors. While DBus may be 
-                    # available, NetworkManager doesn't necessarily have to be
-                    mutter("unable to get networkmanager state: %r" % e)
-                
-        # Create the file list
-        self._create_file_view()
-        # Create the pending merges
-        self._create_pending_merges()
-        
-        # Expand the corresponding expander
-        if self._is_pending:
-            self._expander_merges.set_expanded(True)
-        else:
-            self._expander_files.set_expanded(True)
-        
-        # Display dialog
-        self.vbox.show_all()
-        
-        # Default to Commit button
         self._button_commit.grab_default()
-    
-    def _on_treeview_files_row_activated(self, treeview, path, view_column):
-        # FIXME: the diff window freezes for some reason
+
+    def _add_to_right_table(self, widget, weight, expanding=False):
+        """Add another widget to the table
+
+        :param widget: The object to add
+        :param weight: How many rows does this widget get to request
+        :param expanding: Should expand|fill|shrink be set?
+        """
+        end_row = self._right_pane_table_row + weight
+        options = 0
+        expand_opts = gtk.EXPAND | gtk.FILL | gtk.SHRINK
+        if expanding:
+            options = expand_opts
+        self._right_pane_table.attach(widget, 0, 1,
+                                      self._right_pane_table_row, end_row,
+                                      xoptions=expand_opts, yoptions=options)
+        self._right_pane_table_row = end_row
+
+    def _construct_file_list(self):
+        self._files_box = gtk.VBox(homogeneous=False, spacing=0)
+        file_label = gtk.Label(_('Files'))
+        # file_label.show()
+        self._files_box.pack_start(file_label, expand=False)
+
+        self._commit_all_files_radio = gtk.RadioButton(
+            None, _("Commit all changes"))
+        self._files_box.pack_start(self._commit_all_files_radio, expand=False)
+        self._commit_all_files_radio.show()
+        self._commit_all_files_radio.connect('toggled',
+            self._toggle_commit_selection)
+        self._commit_selected_radio = gtk.RadioButton(
+            self._commit_all_files_radio, _("Only commit selected changes"))
+        self._files_box.pack_start(self._commit_selected_radio, expand=False)
+        self._commit_selected_radio.show()
+        self._commit_selected_radio.connect('toggled',
+            self._toggle_commit_selection)
+        if self._pending:
+            self._commit_all_files_radio.set_label(_('Commit all changes*'))
+            self._commit_all_files_radio.set_sensitive(False)
+            self._commit_selected_radio.set_sensitive(False)
+
+        scroller = gtk.ScrolledWindow()
+        scroller.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
+        self._treeview_files = gtk.TreeView()
+        self._treeview_files.show()
+        scroller.add(self._treeview_files)
+        scroller.set_shadow_type(gtk.SHADOW_IN)
+        scroller.show()
+        self._files_box.pack_start(scroller,
+                                   expand=True, fill=True)
+        self._files_box.show()
+        self._left_pane_box.pack_start(self._files_box)
+
+        # Keep note that all strings stored in a ListStore must be UTF-8
+        # strings. GTK does not support directly setting and restoring Unicode
+        # objects.
+        liststore = gtk.ListStore(
+            gobject.TYPE_STRING,  # [0] file_id
+            gobject.TYPE_STRING,  # [1] real path
+            gobject.TYPE_BOOLEAN, # [2] checkbox
+            gobject.TYPE_STRING,  # [3] display path
+            gobject.TYPE_STRING,  # [4] changes type
+            gobject.TYPE_STRING,  # [5] commit message
+            )
+        self._files_store = liststore
+        self._treeview_files.set_model(liststore)
+        crt = gtk.CellRendererToggle()
+        crt.set_property('activatable', not bool(self._pending))
+        crt.connect("toggled", self._toggle_commit, self._files_store)
+        if self._pending:
+            name = _('Commit*')
+        else:
+            name = _('Commit')
+        commit_col = gtk.TreeViewColumn(name, crt, active=2)
+        commit_col.set_visible(False)
+        self._treeview_files.append_column(commit_col)
+        self._treeview_files.append_column(gtk.TreeViewColumn(_('Path'),
+                                           gtk.CellRendererText(), text=3))
+        self._treeview_files.append_column(gtk.TreeViewColumn(_('Type'),
+                                           gtk.CellRendererText(), text=4))
+        self._treeview_files.connect('cursor-changed',
+                                     self._on_treeview_files_cursor_changed)
+
+    def _toggle_commit(self, cell, path, model):
+        if model[path][0] is None: # No file_id means 'All Files'
+            new_val = not model[path][2]
+            for node in model:
+                node[2] = new_val
+        else:
+            model[path][2] = not model[path][2]
+
+    def _toggle_commit_selection(self, button):
+        all_files = self._commit_all_files_radio.get_active()
+        if self._commit_all_changes != all_files:
+            checked_col = self._treeview_files.get_column(0)
+            self._commit_all_changes = all_files
+            if all_files:
+                checked_col.set_visible(False)
+            else:
+                checked_col.set_visible(True)
+            renderer = checked_col.get_cell_renderers()[0]
+            renderer.set_property('activatable', not all_files)
+
+    def _construct_pending_list(self):
+        # Pending information defaults to hidden, we put it all in 1 box, so
+        # that we can show/hide all of them at once
+        self._pending_box = gtk.VBox()
+        self._pending_box.hide()
+
+        pending_message = gtk.Label()
+        pending_message.set_markup(
+            _('<i>* Cannot select specific files when merging</i>'))
+        self._pending_box.pack_start(pending_message, expand=False, padding=5)
+        pending_message.show()
+
+        pending_label = gtk.Label(_('Pending Revisions'))
+        self._pending_box.pack_start(pending_label, expand=False, padding=0)
+        pending_label.show()
+
+        scroller = gtk.ScrolledWindow()
+        scroller.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
+        self._treeview_pending = gtk.TreeView()
+        scroller.add(self._treeview_pending)
+        scroller.set_shadow_type(gtk.SHADOW_IN)
+        scroller.show()
+        self._pending_box.pack_start(scroller,
+                                     expand=True, fill=True, padding=5)
+        self._treeview_pending.show()
+        self._left_pane_box.pack_start(self._pending_box)
+
+        liststore = gtk.ListStore(gobject.TYPE_STRING, # revision_id
+                                  gobject.TYPE_STRING, # date
+                                  gobject.TYPE_STRING, # committer
+                                  gobject.TYPE_STRING, # summary
+                                 )
+        self._pending_store = liststore
+        self._treeview_pending.set_model(liststore)
+        self._treeview_pending.append_column(gtk.TreeViewColumn(_('Date'),
+                                             gtk.CellRendererText(), text=1))
+        self._treeview_pending.append_column(gtk.TreeViewColumn(_('Committer'),
+                                             gtk.CellRendererText(), text=2))
+        self._treeview_pending.append_column(gtk.TreeViewColumn(_('Summary'),
+                                             gtk.CellRendererText(), text=3))
+
+    def _construct_diff_view(self):
+        from diff import DiffView
+
+        # TODO: jam 2007-10-30 The diff label is currently disabled. If we
+        #       decide that we really don't ever want to display it, we should
+        #       actually remove it, and other references to it, along with the
+        #       tests that it is set properly.
+        self._diff_label = gtk.Label(_('Diff for whole tree'))
+        self._diff_label.set_alignment(0, 0)
+        self._right_pane_table.set_row_spacing(self._right_pane_table_row, 0)
+        self._add_to_right_table(self._diff_label, 1, False)
+        # self._diff_label.show()
+
+        self._diff_view = DiffView()
+        self._add_to_right_table(self._diff_view, 4, True)
+        self._diff_view.show()
+
+    def _construct_file_message(self):
+        scroller = gtk.ScrolledWindow()
+        scroller.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
+
+        self._file_message_text_view = gtk.TextView()
+        scroller.add(self._file_message_text_view)
+        scroller.set_shadow_type(gtk.SHADOW_IN)
+        scroller.show()
+
+        self._file_message_text_view.modify_font(pango.FontDescription("Monospace"))
+        self._file_message_text_view.set_wrap_mode(gtk.WRAP_WORD)
+        self._file_message_text_view.set_accepts_tab(False)
+        self._file_message_text_view.show()
+
+        self._file_message_expander = gtk.Expander(_('File commit message'))
+        self._file_message_expander.set_expanded(True)
+        self._file_message_expander.add(scroller)
+        self._add_to_right_table(self._file_message_expander, 1, False)
+        self._file_message_expander.show()
+
+    def _construct_global_message(self):
+        self._global_message_label = gtk.Label(_('Global Commit Message'))
+        self._global_message_label.set_markup(_('<b>Global Commit Message</b>'))
+        self._global_message_label.set_alignment(0, 0)
+        self._right_pane_table.set_row_spacing(self._right_pane_table_row, 0)
+        self._add_to_right_table(self._global_message_label, 1, False)
+        # Can we remove the spacing between the label and the box?
+        self._global_message_label.show()
+
+        scroller = gtk.ScrolledWindow()
+        scroller.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
+
+        self._global_message_text_view = gtk.TextView()
+        self._global_message_text_view.modify_font(pango.FontDescription("Monospace"))
+        scroller.add(self._global_message_text_view)
+        scroller.set_shadow_type(gtk.SHADOW_IN)
+        scroller.show()
+        self._add_to_right_table(scroller, 2, True)
+        self._file_message_text_view.set_wrap_mode(gtk.WRAP_WORD)
+        self._file_message_text_view.set_accepts_tab(False)
+        self._global_message_text_view.show()
+
+    def _on_treeview_files_cursor_changed(self, treeview):
         treeselection = treeview.get_selection()
-        (model, iter) = treeselection.get_selected()
-        
-        if iter is not None:
-            from diff import DiffWindow
-            
-            _selected = model.get_value(iter, 1)
-            
-            diff = DiffWindow()
-            diff.set_type_hint(gtk.gdk.WINDOW_TYPE_HINT_DIALOG)
-            diff.set_modal(True)
-            parent_tree = self.wt.branch.repository.revision_tree(self.wt.branch.last_revision())
-            diff.set_diff(self.wt.branch.nick, self.wt, parent_tree)
-            try:
-                diff.set_file(_selected)
-            except errors.NoSuchFile:
-                pass
-            diff.show()
-    
+        (model, selection) = treeselection.get_selected()
+
+        if selection is not None:
+            path, display_path = model.get(selection, 1, 3)
+            self._diff_label.set_text(_('Diff for ') + display_path)
+            if path is None:
+                self._diff_view.show_diff(None)
+            else:
+                self._diff_view.show_diff([path.decode('UTF-8')])
+            self._update_per_file_info(selection)
+
+    def _on_accel_next(self, accel_group, window, keyval, modifier):
+        # We don't really care about any of the parameters, because we know
+        # where this message came from
+        tree_selection = self._treeview_files.get_selection()
+        (model, selection) = tree_selection.get_selected()
+        if selection is None:
+            next = None
+        else:
+            next = model.iter_next(selection)
+
+        if next is None:
+            # We have either made it to the end of the list, or nothing was
+            # selected. Either way, select All Files, and jump to the global
+            # commit message.
+            self._treeview_files.set_cursor((0,))
+            self._global_message_text_view.grab_focus()
+        else:
+            # Set the cursor to this entry, and jump to the per-file commit
+            # message
+            self._treeview_files.set_cursor(model.get_path(next))
+            self._file_message_text_view.grab_focus()
+
+    def _save_current_file_message(self):
+        if self._last_selected_file is None:
+            return # Nothing to save
+        text_buffer = self._file_message_text_view.get_buffer()
+        cur_text = text_buffer.get_text(text_buffer.get_start_iter(),
+                                        text_buffer.get_end_iter())
+        last_selected = self._files_store.get_iter(self._last_selected_file)
+        self._files_store.set_value(last_selected, 5, cur_text)
+
+    def _update_per_file_info(self, selection):
+        # The node is changing, so cache the current message
+        if not self._enable_per_file_commits:
+            return
+
+        self._save_current_file_message()
+        text_buffer = self._file_message_text_view.get_buffer()
+        file_id, display_path, message = self._files_store.get(selection, 0, 3, 5)
+        if file_id is None: # Whole tree
+            self._file_message_expander.set_label(_('File commit message'))
+            self._file_message_expander.set_expanded(False)
+            self._file_message_expander.set_sensitive(False)
+            text_buffer.set_text('')
+            self._last_selected_file = None
+        else:
+            self._file_message_expander.set_label(_('Commit message for ')
+                                                  + display_path)
+            self._file_message_expander.set_expanded(True)
+            self._file_message_expander.set_sensitive(True)
+            text_buffer.set_text(message)
+            self._last_selected_file = self._files_store.get_path(selection)
+
+    def _get_specific_files(self):
+        """Return the list of selected paths, and file info.
+
+        :return: ([unicode paths], [{utf-8 file info}]
+        """
+        self._save_current_file_message()
+        files = []
+        records = iter(self._files_store)
+        rec = records.next() # Skip the All Files record
+        assert rec[0] is None, "Are we skipping the wrong record?"
+
+        file_info = []
+        for record in records:
+            if self._commit_all_changes or record[2]:# [2] checkbox
+                file_id = record[0] # [0] file_id
+                path = record[1]    # [1] real path
+                file_message = record[5] # [5] commit message
+                files.append(path.decode('UTF-8'))
+                if self._enable_per_file_commits and file_message:
+                    # All of this needs to be utf-8 information
+                    file_info.append({'path':path, 'file_id':file_id,
+                                     'message':file_message})
+        file_info.sort(key=lambda x:(x['path'], x['file_id']))
+        if self._enable_per_file_commits:
+            return files, file_info
+        else:
+            return files, []
+
     @show_bzr_error
     def _on_commit_clicked(self, button):
         """ Commit button clicked handler. """
-        textbuffer = self._textview_message.get_buffer()
-        start, end = textbuffer.get_bounds()
-        message = textbuffer.get_text(start, end).decode('utf-8')
-        
-        if not self.pending:
-            specific_files = self._get_specific_files()
-        else:
-            specific_files = None
+        self._do_commit()
+
+    def _do_commit(self):
+        message = self._get_global_commit_message()
 
         if message == '':
-            response = question_dialog(_('Commit with an empty message?'),
-                                       _('You can describe your commit intent in the message.'))
+            response = self._question_dialog(
+                            _('Commit with an empty message?'),
+                            _('You can describe your commit intent in the message.'))
             if response == gtk.RESPONSE_NO:
                 # Kindly give focus to message area
-                self._textview_message.grab_focus()
+                self._global_message_text_view.grab_focus()
                 return
 
-        if self._is_checkout:
-            local = self._check_local.get_active()
-        else:
-            local = False
-
-        if list(self.wt.unknowns()) != []:
-            response = question_dialog(_("Commit with unknowns?"),
-               _("Unknown files exist in the working tree. Commit anyway?"))
+        specific_files, file_info = self._get_specific_files()
+        if self._pending:
+            specific_files = None
+
+        local = self._check_local.get_active()
+
+        # All we care about is if there is a single unknown, so if this loop is
+        # entered, then there are unknown files.
+        # TODO: jam 20071002 It seems like this should cancel the dialog
+        #       entirely, since there isn't a way for them to add the unknown
+        #       files at this point.
+        for path in self._wt.unknowns():
+            response = self._question_dialog(
+                _("Commit with unknowns?"),
+                _("Unknown files exist in the working tree. Commit anyway?"))
             if response == gtk.RESPONSE_NO:
                 return
-        
+            break
+
+        rev_id = None
+        revprops = {}
+        if file_info:
+            revprops['file-info'] = bencode.bencode(file_info).decode('UTF-8')
         try:
-            self.wt.commit(message,
+            rev_id = self._wt.commit(message,
                        allow_pointless=False,
                        strict=False,
                        local=local,
-                       specific_files=specific_files)
+                       specific_files=specific_files,
+                       revprops=revprops)
         except errors.PointlessCommit:
-            response = question_dialog(_('Commit with no changes?'),
-                                       _('There are no changes in the working tree.'))
+            response = self._question_dialog(
+                                _('Commit with no changes?'),
+                                _('There are no changes in the working tree.'
+                                  ' Do you want to commit anyway?'))
             if response == gtk.RESPONSE_YES:
-                self.wt.commit(message,
+                rev_id = self._wt.commit(message,
                                allow_pointless=True,
                                strict=False,
                                local=local,
-                               specific_files=specific_files)
+                               specific_files=specific_files,
+                               revprops=revprops)
+        self.committed_revision_id = rev_id
         self.response(gtk.RESPONSE_OK)
 
-    def _pending_merges(self, wt):
-        """ Return a list of pending merges or None if there are none of them. """
-        parents = wt.get_parent_ids()
-        if len(parents) < 2:
-            return None
-        
-        import re
+    def _get_global_commit_message(self):
+        buf = self._global_message_text_view.get_buffer()
+        start, end = buf.get_bounds()
+        return buf.get_text(start, end).decode('utf-8')
+
+    def _set_global_commit_message(self, message):
+        """Just a helper for the test suite."""
+        if isinstance(message, unicode):
+            message = message.encode('UTF-8')
+        self._global_message_text_view.get_buffer().set_text(message)
+
+    def _set_file_commit_message(self, message):
+        """Helper for the test suite."""
+        if isinstance(message, unicode):
+            message = message.encode('UTF-8')
+        self._file_message_text_view.get_buffer().set_text(message)
+
+    @staticmethod
+    def _rev_to_pending_info(rev):
+        """Get the information from a pending merge."""
         from bzrlib.osutils import format_date
-        
-        pending = parents[1:]
-        branch = wt.branch
-        last_revision = parents[0]
-        
-        if last_revision is not None:
-            try:
-                ignore = set(branch.repository.get_ancestry(last_revision))
-            except errors.NoSuchRevision:
-                # the last revision is a ghost : assume everything is new 
-                # except for it
-                ignore = set([None, last_revision])
-        else:
-            ignore = set([None])
-        
-        pm = []
-        for merge in pending:
-            ignore.add(merge)
-            try:
-                m_revision = branch.repository.get_revision(merge)
-                
-                rev = {}
-                rev['committer'] = re.sub('<.*@.*>', '', m_revision.committer).strip(' ')
-                rev['summary'] = m_revision.get_summary()
-                rev['date'] = format_date(m_revision.timestamp,
-                                          m_revision.timezone or 0, 
-                                          'original', date_fmt="%Y-%m-%d",
-                                          show_offset=False)
-                
-                pm.append(rev)
-                
-                inner_merges = branch.repository.get_ancestry(merge)
-                assert inner_merges[0] is None
-                inner_merges.pop(0)
-                inner_merges.reverse()
-                for mmerge in inner_merges:
-                    if mmerge in ignore:
-                        continue
-                    mm_revision = branch.repository.get_revision(mmerge)
-                    
-                    rev = {}
-                    rev['committer'] = re.sub('<.*@.*>', '', mm_revision.committer).strip(' ')
-                    rev['summary'] = mm_revision.get_summary()
-                    rev['date'] = format_date(mm_revision.timestamp,
-                                              mm_revision.timezone or 0, 
-                                              'original', date_fmt="%Y-%m-%d",
-                                              show_offset=False)
-                
-                    pm.append(rev)
-                    
-                    ignore.add(mmerge)
-            except errors.NoSuchRevision:
-                print "DEBUG: NoSuchRevision:", merge
-        
-        return pm
-
-    def _create_file_view(self):
-        self._file_store = gtk.ListStore(gobject.TYPE_BOOLEAN,   # [0] checkbox
-                                         gobject.TYPE_STRING,    # [1] path to display
-                                         gobject.TYPE_STRING,    # [2] changes type
-                                         gobject.TYPE_STRING)    # [3] real path
-        self._treeview_files.set_model(self._file_store)
-        crt = gtk.CellRendererToggle()
-        crt.set_property("activatable", True)
-        crt.connect("toggled", self._toggle_commit, self._file_store)
-        commit_column = gtk.TreeViewColumn(_('Commit'), crt, active=0)
-        if self._is_pending:
-            commit_column.set_visible(False)
-        self._treeview_files.append_column(commit_column)
-        self._treeview_files.append_column(gtk.TreeViewColumn(_('Path'),
-                                     gtk.CellRendererText(), text=1))
-        self._treeview_files.append_column(gtk.TreeViewColumn(_('Type'),
-                                     gtk.CellRendererText(), text=2))
-
-        for path, id, kind in self.delta.added:
-            marker = osutils.kind_marker(kind)
-            if self.selected is not None:
-                if path == os.path.join(self.wtpath, self.selected):
-                    self._file_store.append([ True, path+marker, _('added'), path ])
-                else:
-                    self._file_store.append([ False, path+marker, _('added'), path ])
-            else:
-                self._file_store.append([ True, path+marker, _('added'), path ])
-
-        for path, id, kind in self.delta.removed:
-            marker = osutils.kind_marker(kind)
-            if self.selected is not None:
-                if path == os.path.join(self.wtpath, self.selected):
-                    self._file_store.append([ True, path+marker, _('removed'), path ])
-                else:
-                    self._file_store.append([ False, path+marker, _('removed'), path ])
-            else:
-                self._file_store.append([ True, path+marker, _('removed'), path ])
-
-        for oldpath, newpath, id, kind, text_modified, meta_modified in self.delta.renamed:
-            marker = osutils.kind_marker(kind)
-            if text_modified or meta_modified:
-                changes = _('renamed and modified')
-            else:
-                changes = _('renamed')
-            if self.selected is not None:
-                if newpath == os.path.join(self.wtpath, self.selected):
-                    self._file_store.append([ True,
-                                              oldpath+marker + '  =>  ' + newpath+marker,
-                                              changes,
-                                              newpath
-                                            ])
-                else:
-                    self._file_store.append([ False,
-                                              oldpath+marker + '  =>  ' + newpath+marker,
-                                              changes,
-                                              newpath
-                                            ])
-            else:
-                self._file_store.append([ True,
-                                          oldpath+marker + '  =>  ' + newpath+marker,
-                                          changes,
-                                          newpath
-                                        ])
-
-        for path, id, kind, text_modified, meta_modified in self.delta.modified:
-            marker = osutils.kind_marker(kind)
-            if self.selected is not None:
-                if path == os.path.join(self.wtpath, self.selected):
-                    self._file_store.append([ True, path+marker, _('modified'), path ])
-                else:
-                    self._file_store.append([ False, path+marker, _('modified'), path ])
-            else:
-                self._file_store.append([ True, path+marker, _('modified'), path ])
-    
-    def _create_pending_merges(self):
-        if not self.pending:
-            return
-        
-        liststore = gtk.ListStore(gobject.TYPE_STRING,
-                                  gobject.TYPE_STRING,
-                                  gobject.TYPE_STRING)
-        self._treeview_merges.set_model(liststore)
-        
-        self._treeview_merges.append_column(gtk.TreeViewColumn(_('Date'),
-                                            gtk.CellRendererText(), text=0))
-        self._treeview_merges.append_column(gtk.TreeViewColumn(_('Committer'),
-                                            gtk.CellRendererText(), text=1))
-        self._treeview_merges.append_column(gtk.TreeViewColumn(_('Summary'),
-                                            gtk.CellRendererText(), text=2))
-        
-        for item in self.pending:
-            liststore.append([ item['date'],
-                               item['committer'],
-                               item['summary'] ])
-    
-    def _get_specific_files(self):
-        ret = []
-        it = self._file_store.get_iter_first()
-        while it:
-            if self._file_store.get_value(it, 0):
-                # get real path from hidden column 3
-                ret.append(self._file_store.get_value(it, 3))
-            it = self._file_store.iter_next(it)
-
-        return ret
-    
-    def _toggle_commit(self, cell, path, model):
-        model[path][0] = not model[path][0]
-        return
+        rev_dict = {}
+        rev_dict['committer'] = re.sub('<.*@.*>', '', rev.committer).strip(' ')
+        rev_dict['summary'] = rev.get_summary()
+        rev_dict['date'] = format_date(rev.timestamp,
+                                       rev.timezone or 0,
+                                       'original', date_fmt="%Y-%m-%d",
+                                       show_offset=False)
+        rev_dict['revision_id'] = rev.revision_id
+        return rev_dict

=== modified file 'diff.py'
--- diff.py	2007-11-03 17:41:13 +0000
+++ diff.py	2007-11-07 14:19:09 +0000
@@ -30,72 +30,26 @@
 except ImportError:
     have_gconf = False
 
-import bzrlib
-
-from bzrlib.diff import show_diff_trees
+from bzrlib import osutils
+from bzrlib.diff import show_diff_trees, internal_diff
 from bzrlib.errors import NoSuchFile
 from bzrlib.trace import warning
 from bzrlib.plugins.gtk.window import Window
 
-class DiffWindow(Window):
-    """Diff window.
-
-    This object represents and manages a single window containing the
-    differences between two revisions on a branch.
-    """
-
-    def __init__(self, parent=None):
-        Window.__init__(self, parent)
-        self.set_border_width(0)
-        self.set_title("Changes")
-
-        # Use two thirds of the screen by default
-        screen = self.get_screen()
-        monitor = screen.get_monitor_geometry(0)
-        width = int(monitor.width * 0.66)
-        height = int(monitor.height * 0.66)
-        self.set_default_size(width, height)
+
+class DiffView(gtk.ScrolledWindow):
+    """This is the soft and chewy filling for a DiffWindow."""
+
+    def __init__(self):
+        gtk.ScrolledWindow.__init__(self)
 
         self.construct()
+        self.rev_tree = None
+        self.parent_tree = None
 
     def construct(self):
-        """Construct the window contents."""
-        # The   window  consists  of   a  pane   containing:  the
-        # hierarchical list  of files on  the left, and  the diff
-        # for the currently selected file on the right.
-        pane = gtk.HPaned()
-        self.add(pane)
-        pane.show()
-
-        # The file hierarchy: a scrollable treeview
-        scrollwin = gtk.ScrolledWindow()
-        scrollwin.set_policy(gtk.POLICY_NEVER, gtk.POLICY_AUTOMATIC)
-        scrollwin.set_shadow_type(gtk.SHADOW_IN)
-        pane.pack1(scrollwin)
-        scrollwin.show()
-
-        self.model = gtk.TreeStore(str, str)
-        self.treeview = gtk.TreeView(self.model)
-        self.treeview.set_headers_visible(False)
-        self.treeview.set_search_column(1)
-        self.treeview.connect("cursor-changed", self._treeview_cursor_cb)
-        scrollwin.add(self.treeview)
-        self.treeview.show()
-
-        cell = gtk.CellRendererText()
-        cell.set_property("width-chars", 20)
-        column = gtk.TreeViewColumn()
-        column.pack_start(cell, expand=True)
-        column.add_attribute(cell, "text", 0)
-        self.treeview.append_column(column)
-
-        # The diffs of the  selected file: a scrollable source or
-        # text view
-        scrollwin = gtk.ScrolledWindow()
-        scrollwin.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
-        scrollwin.set_shadow_type(gtk.SHADOW_IN)
-        pane.pack2(scrollwin)
-        scrollwin.show()
+        self.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
+        self.set_shadow_type(gtk.SHADOW_IN)
 
         if have_gtksourceview:
             self.buffer = gtksourceview.SourceBuffer()
@@ -114,78 +68,15 @@
 
         sourceview.set_editable(False)
         sourceview.modify_font(pango.FontDescription("Monospace"))
-        scrollwin.add(sourceview)
+        self.add(sourceview)
         sourceview.show()
 
-    def set_diff(self, description, rev_tree, parent_tree):
-        """Set the differences showed by this window.
-
-        Compares the two trees and populates the window with the
-        differences.
-        """
-        self.rev_tree = rev_tree
-        self.parent_tree = parent_tree
-
-        self.model.clear()
-        delta = self.rev_tree.changes_from(self.parent_tree)
-
-        self.model.append(None, [ "All Changes", "" ])
-
-        if len(delta.added):
-            titer = self.model.append(None, [ "Added", None ])
-            for path, id, kind in delta.added:
-                self.model.append(titer, [ path, path ])
-
-        if len(delta.removed):
-            titer = self.model.append(None, [ "Removed", None ])
-            for path, id, kind in delta.removed:
-                self.model.append(titer, [ path, path ])
-
-        if len(delta.renamed):
-            titer = self.model.append(None, [ "Renamed", None ])
-            for oldpath, newpath, id, kind, text_modified, meta_modified \
-                    in delta.renamed:
-                self.model.append(titer, [ oldpath, newpath ])
-
-        if len(delta.modified):
-            titer = self.model.append(None, [ "Modified", None ])
-            for path, id, kind, text_modified, meta_modified in delta.modified:
-                self.model.append(titer, [ path, path ])
-
-        self.treeview.expand_all()
-        self.set_title(description + " - Changes")
-
-    def set_file(self, file_path):
-        tv_path = None
-        for data in self.model:
-            for child in data.iterchildren():
-                if child[0] == file_path or child[1] == file_path:
-                    tv_path = child.path
-                    break
-        if tv_path is None:
-            raise NoSuchFile(file_path)
-        self.treeview.set_cursor(tv_path)
-        self.treeview.scroll_to_cell(tv_path)
-
-    def _treeview_cursor_cb(self, *args):
-        """Callback for when the treeview cursor changes."""
-        (path, col) = self.treeview.get_cursor()
-        specific_files = [ self.model[path][1] ]
-        if specific_files == [ None ]:
-            return
-        elif specific_files == [ "" ]:
-            specific_files = None
-
-        s = StringIO()
-        show_diff_trees(self.parent_tree, self.rev_tree, s, specific_files)
-        self.buffer.set_text(s.getvalue().decode(sys.getdefaultencoding(), 'replace'))
-
     @staticmethod
     def apply_gedit_colors(lang):
         """Set style for lang to that specified in gedit configuration.
 
         This method needs the gconf module.
-        
+
         :param lang: a gtksourceview.SourceLanguage object.
         """
         GEDIT_SYNTAX_PATH = '/apps/gedit-2/preferences/syntax_highlighting'
@@ -261,7 +152,7 @@
                 except IOError, e:
                     warning('could not open file %s: %s' % (f, str(e)))
                 else:
-                    colors.update(DiffWindow.parse_colordiffrc(f))
+                    colors.update(DiffView.parse_colordiffrc(f))
                     f.close()
 
         if not colors:
@@ -303,7 +194,7 @@
     @staticmethod
     def parse_colordiffrc(fileobj):
         """Parse fileobj as a colordiff configuration file.
-        
+
         :return: A dict with the key -> value pairs.
         """
         colors = {}
@@ -316,3 +207,260 @@
             colors[key.strip()] = val.strip()
         return colors
 
+    def set_trees(self, rev_tree, parent_tree):
+        self.rev_tree = rev_tree
+        self.parent_tree = parent_tree
+#        self._build_delta()
+
+#    def _build_delta(self):
+#        self.parent_tree.lock_read()
+#        self.rev_tree.lock_read()
+#        try:
+#            self.delta = _iter_changes_to_status(self.parent_tree, self.rev_tree)
+#            self.path_to_status = {}
+#            self.path_to_diff = {}
+#            source_inv = self.parent_tree.inventory
+#            target_inv = self.rev_tree.inventory
+#            for (file_id, real_path, change_type, display_path) in self.delta:
+#                self.path_to_status[real_path] = u'=== %s %s' % (change_type, display_path)
+#                if change_type in ('modified', 'renamed and modified'):
+#                    source_ie = source_inv[file_id]
+#                    target_ie = target_inv[file_id]
+#                    sio = StringIO()
+#                    source_ie.diff(internal_diff, *old path, *old_tree,
+#                                   *new_path, target_ie, self.rev_tree,
+#                                   sio)
+#                    self.path_to_diff[real_path] = 
+#
+#        finally:
+#            self.rev_tree.unlock()
+#            self.parent_tree.unlock()
+
+    def show_diff(self, specific_files):
+        s = StringIO()
+        show_diff_trees(self.parent_tree, self.rev_tree, s, specific_files,
+                        old_label='', new_label='',
+                        # path_encoding=sys.getdefaultencoding()
+                        # The default is utf-8, but we interpret the file
+                        # contents as getdefaultencoding(), so we should
+                        # probably try to make the paths in the same encoding.
+                        )
+        # str.decode(encoding, 'replace') doesn't do anything. Because if a
+        # character is not valid in 'encoding' there is nothing to replace, the
+        # 'replace' is for 'str.encode()'
+        try:
+            decoded = s.getvalue().decode(sys.getdefaultencoding())
+        except UnicodeDecodeError:
+            try:
+                decoded = s.getvalue().decode('UTF-8')
+            except UnicodeDecodeError:
+                decoded = s.getvalue().decode('iso-8859-1')
+                # This always works, because every byte has a valid
+                # mapping from iso-8859-1 to Unicode
+        # TextBuffer must contain pure UTF-8 data
+        self.buffer.set_text(decoded.encode('UTF-8'))
+
+
+class DiffWindow(Window):
+    """Diff window.
+
+    This object represents and manages a single window containing the
+    differences between two revisions on a branch.
+    """
+
+    def __init__(self, parent=None):
+        Window.__init__(self, parent)
+        self.set_border_width(0)
+        self.set_title("bzrk diff")
+
+        # Use two thirds of the screen by default
+        screen = self.get_screen()
+        monitor = screen.get_monitor_geometry(0)
+        width = int(monitor.width * 0.66)
+        height = int(monitor.height * 0.66)
+        self.set_default_size(width, height)
+
+        self.construct()
+
+    def construct(self):
+        """Construct the window contents."""
+        # The   window  consists  of   a  pane   containing:  the
+        # hierarchical list  of files on  the left, and  the diff
+        # for the currently selected file on the right.
+        pane = gtk.HPaned()
+        self.add(pane)
+        pane.show()
+
+        # The file hierarchy: a scrollable treeview
+        scrollwin = gtk.ScrolledWindow()
+        scrollwin.set_policy(gtk.POLICY_NEVER, gtk.POLICY_AUTOMATIC)
+        scrollwin.set_shadow_type(gtk.SHADOW_IN)
+        pane.pack1(scrollwin)
+        scrollwin.show()
+
+        self.model = gtk.TreeStore(str, str)
+        self.treeview = gtk.TreeView(self.model)
+        self.treeview.set_headers_visible(False)
+        self.treeview.set_search_column(1)
+        self.treeview.connect("cursor-changed", self._treeview_cursor_cb)
+        scrollwin.add(self.treeview)
+        self.treeview.show()
+
+        cell = gtk.CellRendererText()
+        cell.set_property("width-chars", 20)
+        column = gtk.TreeViewColumn()
+        column.pack_start(cell, expand=True)
+        column.add_attribute(cell, "text", 0)
+        self.treeview.append_column(column)
+
+        # The diffs of the  selected file: a scrollable source or
+        # text view
+        self.diff_view = DiffView()
+        pane.pack2(self.diff_view)
+        self.diff_view.show()
+
+    def set_diff(self, description, rev_tree, parent_tree):
+        """Set the differences showed by this window.
+
+        Compares the two trees and populates the window with the
+        differences.
+        """
+        self.diff_view.set_trees(rev_tree, parent_tree)
+        self.rev_tree = rev_tree
+        self.parent_tree = parent_tree
+
+        self.model.clear()
+        delta = self.rev_tree.changes_from(self.parent_tree)
+
+        self.model.append(None, [ "Complete Diff", "" ])
+
+        if len(delta.added):
+            titer = self.model.append(None, [ "Added", None ])
+            for path, id, kind in delta.added:
+                self.model.append(titer, [ path, path ])
+
+        if len(delta.removed):
+            titer = self.model.append(None, [ "Removed", None ])
+            for path, id, kind in delta.removed:
+                self.model.append(titer, [ path, path ])
+
+        if len(delta.renamed):
+            titer = self.model.append(None, [ "Renamed", None ])
+            for oldpath, newpath, id, kind, text_modified, meta_modified \
+                    in delta.renamed:
+                self.model.append(titer, [ oldpath, newpath ])
+
+        if len(delta.modified):
+            titer = self.model.append(None, [ "Modified", None ])
+            for path, id, kind, text_modified, meta_modified in delta.modified:
+                self.model.append(titer, [ path, path ])
+
+        self.treeview.expand_all()
+        self.set_title(description + " - bzrk diff")
+
+    def set_file(self, file_path):
+        tv_path = None
+        for data in self.model:
+            for child in data.iterchildren():
+                if child[0] == file_path or child[1] == file_path:
+                    tv_path = child.path
+                    break
+        if tv_path is None:
+            raise NoSuchFile(file_path)
+        self.treeview.set_cursor(tv_path)
+        self.treeview.scroll_to_cell(tv_path)
+
+    def _treeview_cursor_cb(self, *args):
+        """Callback for when the treeview cursor changes."""
+        (path, col) = self.treeview.get_cursor()
+        specific_files = [ self.model[path][1] ]
+        if specific_files == [ None ]:
+            return
+        elif specific_files == [ "" ]:
+            specific_files = None
+
+        self.diff_view.show_diff(specific_files)
+
+
+def _iter_changes_to_status(source, target):
+    """Determine the differences between trees.
+
+    This is a wrapper around _iter_changes which just yields more
+    understandable results.
+
+    :param source: The source tree (basis tree)
+    :param target: The target tree
+    :return: A list of (file_id, real_path, change_type, display_path)
+    """
+    added = 'added'
+    removed = 'removed'
+    renamed = 'renamed'
+    renamed_and_modified = 'renamed and modified'
+    modified = 'modified'
+    kind_changed = 'kind changed'
+
+    # TODO: Handle metadata changes
+
+    status = []
+    target.lock_read()
+    try:
+        source.lock_read()
+        try:
+            for (file_id, paths, changed_content, versioned, parent_ids, names,
+                 kinds, executables) in target._iter_changes(source):
+
+                # Skip the root entry if it isn't very interesting
+                if parent_ids == (None, None):
+                    continue
+
+                change_type = None
+                if kinds[0] is None:
+                    source_marker = ''
+                else:
+                    source_marker = osutils.kind_marker(kinds[0])
+                if kinds[1] is None:
+                    assert kinds[0] is not None
+                    marker = osutils.kind_marker(kinds[0])
+                else:
+                    marker = osutils.kind_marker(kinds[1])
+
+                real_path = paths[1]
+                if real_path is None:
+                    real_path = paths[0]
+                assert real_path is not None
+                display_path = real_path + marker
+
+                present_source = versioned[0] and kinds[0] is not None
+                present_target = versioned[1] and kinds[1] is not None
+
+                if present_source != present_target:
+                    if present_target:
+                        change_type = added
+                    else:
+                        assert present_source
+                        change_type = removed
+                elif names[0] != names[1] or parent_ids[0] != parent_ids[1]:
+                    # Renamed
+                    if changed_content or executables[0] != executables[1]:
+                        # and modified
+                        change_type = renamed_and_modified
+                    else:
+                        change_type = renamed
+                    display_path = (paths[0] + source_marker
+                                    + ' => ' + paths[1] + marker)
+                elif kinds[0] != kinds[1]:
+                    change_type = kind_changed
+                    display_path = (paths[0] + source_marker
+                                    + ' => ' + paths[1] + marker)
+                elif changed_content is True or executables[0] != executables[1]:
+                    change_type = modified
+                else:
+                    assert False, "How did we get here?"
+
+                status.append((file_id, real_path, change_type, display_path))
+        finally:
+            source.unlock()
+    finally:
+        target.unlock()
+
+    return status

=== modified file 'olive/__init__.py'
--- olive/__init__.py	2007-10-21 16:55:31 +0000
+++ olive/__init__.py	2007-10-26 16:18:50 +0000
@@ -502,7 +502,14 @@
     @show_bzr_error
     def on_menuitem_branch_commit_activate(self, widget):
         """ Branch/Commit... menu handler. """
-        commit = CommitDialog(self.wt, self.wtpath, self.notbranch, self.get_selected_right(), self.window)
+#     def __init__(self, wt, wtpath, notbranch, selected=None, parent=None):
+        selected = self.get_selected_right()
+        if selected:
+            selected = os.path.join(self.wtpath, selected)
+        commit = CommitDialog(wt=self.wt,
+                              parent=self.window,
+                              selected=selected,
+                             )
         response = commit.run()
         if response != gtk.RESPONSE_NONE:
             commit.hide()

=== modified file 'olive/menu.py'
--- olive/menu.py	2007-10-14 15:33:50 +0000
+++ olive/menu.py	2007-10-26 16:18:50 +0000
@@ -288,7 +288,11 @@
         except NotBranchError, e:
             path = e.path
         
-        commit = CommitDialog(wt, path, not branch, self.selected)
+        if self.selected:
+            selected = os.path.join(path, self.selected)
+        else:
+            selected = None
+        commit = CommitDialog(wt=wt, selected=selected, parent=None)
         response = commit.run()
         if response != gtk.RESPONSE_NONE:
             commit.hide()

=== modified file 'revisionview.py'
--- revisionview.py	2007-11-02 14:49:57 +0000
+++ revisionview.py	2007-11-03 16:27:35 +0000
@@ -21,6 +21,7 @@
 import pango
 
 from bzrlib.osutils import format_date
+from bzrlib.util import bencode
 
 class RevisionView(gtk.Notebook):
     """ Custom widget for commit log details.
@@ -36,6 +37,7 @@
 
         self._create_general()
         self._create_relations()
+        self._create_file_info_view()
 
         self.set_current_page(0)
         
@@ -47,6 +49,7 @@
 
         if revision is not None:
             self.set_revision(revision, tags=tags)
+        self.set_file_id(None)
 
     def set_show_callback(self, callback):
         self._show_callback = callback
@@ -54,6 +57,14 @@
     def set_go_callback(self, callback):
         self._go_callback = callback
 
+    def set_file_id(self, file_id):
+        """Set a specific file id that we want to track.
+
+        This just effects the display of a per-file commit message.
+        If it is set to None, then all commit messages will be shown.
+        """
+        self._file_id = file_id
+
     def set_revision(self, revision, tags=[], children=[]):
         self._revision = revision
         self.revision_id.set_text(revision.revision_id)
@@ -90,6 +101,30 @@
         
         self._add_tags(tags)
 
+        file_info = revision.properties.get('file-info', None)
+        if file_info is not None:
+            file_info = bencode.bdecode(file_info.encode('UTF-8'))
+
+        if file_info:
+            if self._file_id is None:
+                text = []
+                for fi in file_info:
+                    text.append('%(path)s\n%(message)s' % fi)
+                self.file_info_buffer.set_text('\n'.join(text))
+                self.file_info_box.show()
+            else:
+                text = []
+                for fi in file_info:
+                    if fi['file_id'] == self._file_id:
+                        text.append(fi['message'])
+                if text:
+                    self.file_info_buffer.set_text('\n'.join(text))
+                    self.file_info_box.show()
+                else:
+                    self.file_info_box.hide()
+        else:
+            self.file_info_box.hide()
+
     def _show_clicked_cb(self, widget, revid, parentid):
         """Callback for when the show button for a parent is clicked."""
         self._show_callback(revid, parentid)
@@ -332,3 +367,21 @@
         window.show()
         return window
 
+    def _create_file_info_view(self):
+        self.file_info_box = gtk.VBox(False, 6)
+        self.file_info_box.set_border_width(6)
+        self.file_info_buffer = gtk.TextBuffer()
+        window = gtk.ScrolledWindow()
+        window.set_policy(gtk.POLICY_NEVER, gtk.POLICY_AUTOMATIC)
+        window.set_shadow_type(gtk.SHADOW_IN)
+        tv = gtk.TextView(self.file_info_buffer)
+        tv.set_editable(False)
+        tv.set_wrap_mode(gtk.WRAP_WORD)
+        tv.modify_font(pango.FontDescription("Monospace"))
+        tv.show()
+        window.add(tv)
+        window.show()
+        self.file_info_box.pack_start(window)
+        self.file_info_box.hide() # Only shown when there are per-file messages
+        self.append_page(self.file_info_box, tab_label=gtk.Label('Per-file'))
+

=== modified file 'setup.py'
--- setup.py	2007-09-25 14:36:11 +0000
+++ setup.py	2007-11-07 10:08:49 +0000
@@ -87,7 +87,7 @@
 
 setup(
     name = "bzr-gtk",
-    version = "0.92.0",
+    version = "0.93.0",
     maintainer = "Jelmer Vernooij",
     maintainer_email = "jelmer@samba.org",
     description = "GTK+ Frontends for various Bazaar commands",

=== modified file 'tests/__init__.py'
--- tests/__init__.py	2007-10-30 20:59:54 +0000
+++ tests/__init__.py	2007-10-30 21:15:13 +0000
@@ -25,6 +25,7 @@
     loader = TestUtil.TestLoader()
 
     testmod_names = [
+        'test_commit',
         'test_diff',
         'test_preferences',
         'test_history',

=== added file 'tests/test_commit.py'
--- tests/test_commit.py	1970-01-01 00:00:00 +0000
+++ tests/test_commit.py	2007-11-02 22:08:54 +0000
@@ -0,0 +1,1033 @@
+# Copyright (C) 2007 John Arbash Meinel <john@arbash-meinel.com>
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""Test the Commit functionality."""
+
+import os
+
+import gtk
+
+from bzrlib import (
+    tests,
+    revision,
+    )
+from bzrlib.util import bencode
+
+from bzrlib.plugins.gtk import commit
+
+
+# TODO: All we need is basic ancestry code to test this, we shouldn't need a
+# TestCaseWithTransport, just a TestCaseWithMemoryTransport or somesuch.
+
+class TestPendingRevisions(tests.TestCaseWithTransport):
+
+    def test_pending_revisions_none(self):
+        tree = self.make_branch_and_tree('.')
+        tree.commit('one')
+
+        self.assertIs(None, commit.pending_revisions(tree))
+
+    def test_pending_revisions_simple(self):
+        tree = self.make_branch_and_tree('tree')
+        rev_id1 = tree.commit('one')
+        tree2 = tree.bzrdir.sprout('tree2').open_workingtree()
+        rev_id2 = tree2.commit('two')
+        tree.merge_from_branch(tree2.branch)
+        self.assertEqual([rev_id1, rev_id2], tree.get_parent_ids())
+
+        pending_revisions = commit.pending_revisions(tree)
+        # One primary merge
+        self.assertEqual(1, len(pending_revisions))
+        # Revision == rev_id2
+        self.assertEqual(rev_id2, pending_revisions[0][0].revision_id)
+        # No children of this revision.
+        self.assertEqual([], pending_revisions[0][1])
+
+    def test_pending_revisions_with_children(self):
+        tree = self.make_branch_and_tree('tree')
+        rev_id1 = tree.commit('one')
+        tree2 = tree.bzrdir.sprout('tree2').open_workingtree()
+        rev_id2 = tree2.commit('two')
+        rev_id3 = tree2.commit('three')
+        rev_id4 = tree2.commit('four')
+        tree.merge_from_branch(tree2.branch)
+        self.assertEqual([rev_id1, rev_id4], tree.get_parent_ids())
+
+        pending_revisions = commit.pending_revisions(tree)
+        # One primary merge
+        self.assertEqual(1, len(pending_revisions))
+        # Revision == rev_id2
+        self.assertEqual(rev_id4, pending_revisions[0][0].revision_id)
+        # Two children for this revision
+        self.assertEqual(2, len(pending_revisions[0][1]))
+        self.assertEqual(rev_id3, pending_revisions[0][1][0].revision_id)
+        self.assertEqual(rev_id2, pending_revisions[0][1][1].revision_id)
+
+    def test_pending_revisions_multi_merge(self):
+        tree = self.make_branch_and_tree('tree')
+        rev_id1 = tree.commit('one')
+        tree2 = tree.bzrdir.sprout('tree2').open_workingtree()
+        rev_id2 = tree2.commit('two')
+        rev_id3 = tree2.commit('three')
+        tree3 = tree2.bzrdir.sprout('tree3').open_workingtree()
+        rev_id4 = tree3.commit('four')
+        rev_id5 = tree3.commit('five')
+        tree.merge_from_branch(tree2.branch)
+        tree.merge_from_branch(tree3.branch)
+        self.assertEqual([rev_id1, rev_id3, rev_id5], tree.get_parent_ids())
+
+        pending_revisions = commit.pending_revisions(tree)
+        # Two primary merges
+        self.assertEqual(2, len(pending_revisions))
+        # Revision == rev_id2
+        self.assertEqual(rev_id3, pending_revisions[0][0].revision_id)
+        self.assertEqual(rev_id5, pending_revisions[1][0].revision_id)
+        # One child for the first merge
+        self.assertEqual(1, len(pending_revisions[0][1]))
+        self.assertEqual(rev_id2, pending_revisions[0][1][0].revision_id)
+        # One child for the second merge
+        self.assertEqual(1, len(pending_revisions[1][1]))
+        self.assertEqual(rev_id4, pending_revisions[1][1][0].revision_id)
+
+
+class Test_RevToPendingInfo(tests.TestCaseWithTransport):
+
+    def test_basic_info(self):
+        tree = self.make_branch_and_tree('tree')
+        rev_id = tree.commit('Multiline\ncommit\nmessage',
+                             committer='Joe Foo <joe@foo.com>',
+                             timestamp=1191012408.674,
+                             timezone=-18000
+                             )
+        rev = tree.branch.repository.get_revision(rev_id)
+        rev_dict = commit.CommitDialog._rev_to_pending_info(rev)
+        self.assertEqual({'committer':'Joe Foo',
+                          'summary':'Multiline',
+                          'date':'2007-09-28',
+                          'revision_id':rev_id,
+                         }, rev_dict)
+
+
+class CommitDialogNoWidgets(commit.CommitDialog):
+
+    def construct(self):
+        pass # Don't create any widgets here
+
+    def fill_in_data(self):
+        pass # With no widgets, there are no widgets to fill out
+
+
+class TestCommitDialogSimple(tests.TestCaseWithTransport):
+
+    def test_setup_parameters_no_pending(self):
+        tree = self.make_branch_and_tree('tree')
+        rev_id = tree.commit('first')
+
+        dlg = CommitDialogNoWidgets(tree)
+        self.assertEqual(rev_id, dlg._basis_tree.get_revision_id())
+        self.assertIs(None, dlg._pending)
+        self.assertFalse(dlg._is_checkout)
+
+    def test_setup_parameters_checkout(self):
+        tree = self.make_branch_and_tree('tree')
+        rev_id = tree.commit('first')
+        tree2 = tree.bzrdir.sprout('tree2').open_workingtree()
+        tree2.branch.bind(tree.branch)
+
+        dlg = CommitDialogNoWidgets(tree2)
+        self.assertEqual(rev_id, dlg._basis_tree.get_revision_id())
+        self.assertIs(None, dlg._pending)
+        self.assertTrue(dlg._is_checkout)
+
+    def test_setup_parameters_pending(self):
+        tree = self.make_branch_and_tree('tree')
+        rev_id1 = tree.commit('one')
+        tree2 = tree.bzrdir.sprout('tree2').open_workingtree()
+        rev_id2 = tree2.commit('two')
+        tree.merge_from_branch(tree2.branch)
+
+        dlg = CommitDialogNoWidgets(tree)
+        self.assertEqual(rev_id1, dlg._basis_tree.get_revision_id())
+        self.assertIsNot(None, dlg._pending)
+        self.assertEqual(1, len(dlg._pending))
+        self.assertEqual(rev_id2, dlg._pending[0][0].revision_id)
+
+    def test_setup_parameters_delta(self):
+        tree = self.make_branch_and_tree('tree')
+        self.build_tree(['tree/a'])
+        tree.add(['a'], ['a-id'])
+
+        dlg = CommitDialogNoWidgets(tree)
+        self.assertIs(None, dlg._delta)
+        dlg._compute_delta()
+
+        delta = dlg._delta
+        self.assertEqual([], delta.modified)
+        self.assertEqual([], delta.renamed)
+        self.assertEqual([], delta.removed)
+        self.assertEqual([(u'a', 'a-id', 'file')], delta.added)
+
+
+class TestCommitDialog(tests.TestCaseWithTransport):
+
+    def test_bound(self):
+        tree = self.make_branch_and_tree('tree')
+        rev_id = tree.commit('first')
+        tree2 = tree.bzrdir.sprout('tree2').open_workingtree()
+        tree2.branch.bind(tree.branch)
+
+        # tree is not a checkout
+        dlg = commit.CommitDialog(tree)
+        self.assertFalse(dlg._check_local.get_property('visible'))
+
+        # tree2 is a checkout
+        dlg2 = commit.CommitDialog(tree2)
+        self.assertTrue(dlg2._check_local.get_property('visible'))
+
+    def test_no_pending(self):
+        tree = self.make_branch_and_tree('tree')
+        rev_id1 = tree.commit('one')
+
+        dlg = commit.CommitDialog(tree)
+
+        self.assertFalse(dlg._pending_box.get_property('visible'))
+
+        commit_col = dlg._treeview_files.get_column(0)
+        self.assertEqual('Commit', commit_col.get_title())
+        renderer = commit_col.get_cell_renderers()[0]
+        self.assertTrue(renderer.get_property('activatable'))
+
+        self.assertEqual('Commit all changes',
+                         dlg._commit_all_files_radio.get_label())
+        self.assertTrue(dlg._commit_all_files_radio.get_property('sensitive'))
+        self.assertTrue(dlg._commit_selected_radio.get_property('sensitive'))
+
+    def test_pending(self):
+        tree = self.make_branch_and_tree('tree')
+        rev_id1 = tree.commit('one')
+
+        tree2 = tree.bzrdir.sprout('tree2').open_workingtree()
+        rev_id2 = tree2.commit('two',
+                               committer='Joe Foo <joe@foo.com>',
+                               timestamp=1191264271.05,
+                               timezone=+7200)
+        tree.merge_from_branch(tree2.branch)
+
+        dlg = commit.CommitDialog(tree)
+
+        self.assertTrue(dlg._pending_box.get_property('visible'))
+
+        commit_col = dlg._treeview_files.get_column(0)
+        self.assertEqual('Commit*', commit_col.get_title())
+        renderer = commit_col.get_cell_renderers()[0]
+        self.assertFalse(renderer.get_property('activatable'))
+
+        values = [(r[0], r[1], r[2], r[3]) for r in dlg._pending_store]
+        self.assertEqual([(rev_id2, '2007-10-01', 'Joe Foo', 'two')], values)
+
+        self.assertEqual('Commit all changes*',
+                         dlg._commit_all_files_radio.get_label())
+        self.assertFalse(dlg._commit_all_files_radio.get_property('sensitive'))
+        self.assertFalse(dlg._commit_selected_radio.get_property('sensitive'))
+
+    def test_pending_multiple(self):
+        tree = self.make_branch_and_tree('tree')
+        rev_id1 = tree.commit('one')
+
+        tree2 = tree.bzrdir.sprout('tree2').open_workingtree()
+        rev_id2 = tree2.commit('two',
+                               committer='Joe Foo <joe@foo.com>',
+                               timestamp=1191264271.05,
+                               timezone=+7200)
+        rev_id3 = tree2.commit('three',
+                               committer='Jerry Foo <jerry@foo.com>',
+                               timestamp=1191264278.05,
+                               timezone=+7200)
+        tree.merge_from_branch(tree2.branch)
+        tree3 = tree.bzrdir.sprout('tree3').open_workingtree()
+        rev_id4 = tree3.commit('four',
+                               committer='Joe Foo <joe@foo.com>',
+                               timestamp=1191264279.05,
+                               timezone=+7200)
+        rev_id5 = tree3.commit('five',
+                               committer='Jerry Foo <jerry@foo.com>',
+                               timestamp=1191372278.05,
+                               timezone=+7200)
+        tree.merge_from_branch(tree3.branch)
+
+        dlg = commit.CommitDialog(tree)
+        # TODO: assert that the pending box is set to show
+        values = [(r[0], r[1], r[2], r[3]) for r in dlg._pending_store]
+        self.assertEqual([(rev_id3, '2007-10-01', 'Jerry Foo', 'three'),
+                          (rev_id2, '2007-10-01', 'Joe Foo', 'two'),
+                          (rev_id5, '2007-10-03', 'Jerry Foo', 'five'),
+                          (rev_id4, '2007-10-01', 'Joe Foo', 'four'),
+                         ], values)
+
+    def test_filelist_added(self):
+        tree = self.make_branch_and_tree('tree')
+        self.build_tree(['tree/a', 'tree/b/', 'tree/b/c'])
+        tree.add(['a', 'b', 'b/c'], ['a-id', 'b-id', 'c-id'])
+
+        dlg = commit.CommitDialog(tree)
+        values = [(r[0], r[1], r[2], r[3], r[4]) for r in dlg._files_store]
+        self.assertEqual([(None, None, True, 'All Files', ''),
+                          ('a-id', 'a', True, 'a', 'added'),
+                          ('b-id', 'b', True, 'b/', 'added'),
+                          ('c-id', 'b/c', True, 'b/c', 'added'),
+                         ], values)
+
+    def test_filelist_renamed(self):
+        tree = self.make_branch_and_tree('tree')
+        self.build_tree(['tree/a', 'tree/b/', 'tree/b/c'])
+        tree.add(['a', 'b', 'b/c'], ['a-id', 'b-id', 'c-id'])
+        rev_id1 = tree.commit('one')
+
+        tree.rename_one('b', 'd')
+        tree.rename_one('a', 'd/a')
+
+        dlg = commit.CommitDialog(tree)
+        values = [(r[0], r[1], r[2], r[3], r[4]) for r in dlg._files_store]
+        self.assertEqual([(None, None, True, 'All Files', ''),
+                          ('b-id', 'd', True, 'b/ => d/', 'renamed'),
+                          ('a-id', 'd/a', True, 'a => d/a', 'renamed'),
+                         ], values)
+
+    def test_filelist_modified(self):
+        tree = self.make_branch_and_tree('tree')
+        self.build_tree(['tree/a'])
+        tree.add(['a'], ['a-id'])
+        rev_id1 = tree.commit('one')
+
+        self.build_tree_contents([('tree/a', 'new contents for a\n')])
+
+        dlg = commit.CommitDialog(tree)
+        values = [(r[0], r[1], r[2], r[3], r[4]) for r in dlg._files_store]
+        self.assertEqual([(None, None, True, 'All Files', ''),
+                          ('a-id', 'a', True, 'a', 'modified'),
+                         ], values)
+
+    def test_filelist_renamed_and_modified(self):
+        tree = self.make_branch_and_tree('tree')
+        self.build_tree(['tree/a', 'tree/b/', 'tree/b/c'])
+        tree.add(['a', 'b', 'b/c'], ['a-id', 'b-id', 'c-id'])
+        rev_id1 = tree.commit('one')
+
+        tree.rename_one('b', 'd')
+        tree.rename_one('a', 'd/a')
+        self.build_tree_contents([('tree/d/a', 'new contents for a\n'),
+                                  ('tree/d/c', 'new contents for c\n'),
+                                 ])
+        # 'c' is not considered renamed, because only its parent was moved, it
+        # stayed in the same directory
+
+        dlg = commit.CommitDialog(tree)
+        values = [(r[0], r[1], r[2], r[3], r[4]) for r in dlg._files_store]
+        self.assertEqual([(None, None, True, 'All Files', ''),
+                          ('b-id', 'd', True, 'b/ => d/', 'renamed'),
+                          ('a-id', 'd/a', True, 'a => d/a', 'renamed and modified'),
+                          ('c-id', 'd/c', True, 'd/c', 'modified'),
+                         ], values)
+
+    def test_filelist_kind_changed(self):
+        tree = self.make_branch_and_tree('tree')
+        self.build_tree(['tree/a', 'tree/b'])
+        tree.add(['a', 'b'], ['a-id', 'b-id'])
+        tree.commit('one')
+
+        os.remove('tree/a')
+        self.build_tree(['tree/a/'])
+        # XXX:  This is technically valid, and the file list handles it fine,
+        #       but 'show_diff_trees()' does not, so we skip this part of the
+        #       test for now.
+        # tree.rename_one('b', 'c')
+        # os.remove('tree/c')
+        # self.build_tree(['tree/c/'])
+
+        dlg = commit.CommitDialog(tree)
+        values = [(r[0], r[1], r[2], r[3], r[4]) for r in dlg._files_store]
+        self.assertEqual([(None, None, True, 'All Files', ''),
+                          ('a-id', 'a', True, 'a => a/', 'kind changed'),
+                          # ('b-id', 'c', True, 'b => c/', 'renamed and modified'),
+                         ], values)
+
+    def test_filelist_removed(self):
+        tree = self.make_branch_and_tree('tree')
+        self.build_tree(['tree/a', 'tree/b/'])
+        tree.add(['a', 'b'], ['a-id', 'b-id'])
+        tree.commit('one')
+
+        os.remove('tree/a')
+        tree.remove('b', force=True)
+
+        dlg = commit.CommitDialog(tree)
+        values = [(r[0], r[1], r[2], r[3], r[4]) for r in dlg._files_store]
+        self.assertEqual([(None, None, True, 'All Files', ''),
+                          ('a-id', 'a', True, 'a', 'removed'),
+                          ('b-id', 'b', True, 'b/', 'removed'),
+                         ], values)
+        # All Files should be selected
+        self.assertEqual(((0,), None), dlg._treeview_files.get_cursor())
+
+    def test_filelist_with_selected(self):
+        tree = self.make_branch_and_tree('tree')
+        self.build_tree(['tree/a', 'tree/b/'])
+        tree.add(['a', 'b'], ['a-id', 'b-id'])
+
+        dlg = commit.CommitDialog(tree, selected='a')
+        values = [(r[0], r[1], r[2], r[3], r[4]) for r in dlg._files_store]
+        self.assertEqual([(None, None, False, 'All Files', ''),
+                          ('a-id', 'a', True, 'a', 'added'),
+                          ('b-id', 'b', False, 'b/', 'added'),
+                         ], values)
+        # This file should also be selected in the file list, rather than the
+        # 'All Files' selection
+        self.assertEqual(((1,), None), dlg._treeview_files.get_cursor())
+
+    def test_diff_view(self):
+        tree = self.make_branch_and_tree('tree')
+        self.build_tree(['tree/a', 'tree/b'])
+        tree.add(['a', 'b'], ['a-id', 'b-id'])
+        tree.commit('one')
+
+        self.build_tree_contents([('tree/a', 'new contents for a\n')])
+        tree.remove('b')
+
+        dlg = commit.CommitDialog(tree)
+        diff_buffer = dlg._diff_view.buffer
+        text = diff_buffer.get_text(diff_buffer.get_start_iter(),
+                                    diff_buffer.get_end_iter()).splitlines(True)
+
+        self.assertEqual("=== removed file 'b'\n", text[0])
+        self.assertContainsRe(text[1],
+            r"--- b\t\d\d\d\d-\d\d-\d\d \d\d:\d\d:\d\d [+-]\d\d\d\d")
+        self.assertEqual('+++ b\t1970-01-01 00:00:00 +0000\n', text[2])
+        self.assertEqual('@@ -1,1 +0,0 @@\n', text[3])
+        self.assertEqual('-contents of tree/b\n', text[4])
+        self.assertEqual('\n', text[5])
+
+        self.assertEqual("=== modified file 'a'\n", text[6])
+        self.assertContainsRe(text[7],
+            r"--- a\t\d\d\d\d-\d\d-\d\d \d\d:\d\d:\d\d [+-]\d\d\d\d")
+        self.assertContainsRe(text[8],
+            r"\+\+\+ a\t\d\d\d\d-\d\d-\d\d \d\d:\d\d:\d\d [+-]\d\d\d\d")
+        self.assertEqual('@@ -1,1 +1,1 @@\n', text[9])
+        self.assertEqual('-contents of tree/a\n', text[10])
+        self.assertEqual('+new contents for a\n', text[11])
+        self.assertEqual('\n', text[12])
+
+        self.assertEqual('Diff for All Files', dlg._diff_label.get_text())
+
+    def test_commit_partial_toggle(self):
+        tree = self.make_branch_and_tree('tree')
+        self.build_tree(['tree/a', 'tree/b'])
+        tree.add(['a', 'b'], ['a-id', 'b-id'])
+
+        dlg = commit.CommitDialog(tree)
+        checked_col = dlg._treeview_files.get_column(0)
+        self.assertFalse(checked_col.get_property('visible'))
+        self.assertTrue(dlg._commit_all_changes)
+
+        dlg._commit_selected_radio.set_active(True)
+        self.assertTrue(checked_col.get_property('visible'))
+        self.assertFalse(dlg._commit_all_changes)
+
+    def test_file_selection(self):
+        """Several things should happen when a file has been selected."""
+        tree = self.make_branch_and_tree('tree')
+        tree.branch.get_config().set_user_option('per_file_commits', 'true')
+        self.build_tree(['tree/a', 'tree/b'])
+        tree.add(['a', 'b'], ['a-id', 'b-id'])
+
+        dlg = commit.CommitDialog(tree)
+        diff_buffer = dlg._diff_view.buffer
+        self.assertEqual('Diff for All Files', dlg._diff_label.get_text())
+        self.assertEqual('File commit message',
+                         dlg._file_message_expander.get_label())
+        self.assertFalse(dlg._file_message_expander.get_expanded())
+        self.assertFalse(dlg._file_message_expander.get_property('sensitive'))
+
+        dlg._treeview_files.set_cursor((1,))
+        self.assertEqual('Diff for a', dlg._diff_label.get_text())
+        text = diff_buffer.get_text(diff_buffer.get_start_iter(),
+                                    diff_buffer.get_end_iter()).splitlines(True)
+        self.assertEqual("=== added file 'a'\n", text[0])
+        self.assertContainsRe(text[1],
+            r"--- a\t\d\d\d\d-\d\d-\d\d \d\d:\d\d:\d\d [+-]\d\d\d\d")
+        self.assertContainsRe(text[2],
+            r"\+\+\+ a\t\d\d\d\d-\d\d-\d\d \d\d:\d\d:\d\d [+-]\d\d\d\d")
+        self.assertEqual('@@ -0,0 +1,1 @@\n', text[3])
+        self.assertEqual('+contents of tree/a\n', text[4])
+        self.assertEqual('\n', text[5])
+        self.assertEqual('Commit message for a',
+                         dlg._file_message_expander.get_label())
+        self.assertTrue(dlg._file_message_expander.get_expanded())
+        self.assertTrue(dlg._file_message_expander.get_property('sensitive'))
+
+        dlg._treeview_files.set_cursor((2,))
+        self.assertEqual('Diff for b', dlg._diff_label.get_text())
+        text = diff_buffer.get_text(diff_buffer.get_start_iter(),
+                                    diff_buffer.get_end_iter()).splitlines(True)
+        self.assertEqual("=== added file 'b'\n", text[0])
+        self.assertContainsRe(text[1],
+            r"--- b\t\d\d\d\d-\d\d-\d\d \d\d:\d\d:\d\d [+-]\d\d\d\d")
+        self.assertContainsRe(text[2],
+            r"\+\+\+ b\t\d\d\d\d-\d\d-\d\d \d\d:\d\d:\d\d [+-]\d\d\d\d")
+        self.assertEqual('@@ -0,0 +1,1 @@\n', text[3])
+        self.assertEqual('+contents of tree/b\n', text[4])
+        self.assertEqual('\n', text[5])
+        self.assertEqual('Commit message for b',
+                         dlg._file_message_expander.get_label())
+        self.assertTrue(dlg._file_message_expander.get_expanded())
+        self.assertTrue(dlg._file_message_expander.get_property('sensitive'))
+
+        dlg._treeview_files.set_cursor((0,))
+        self.assertEqual('Diff for All Files', dlg._diff_label.get_text())
+        self.assertEqual('File commit message',
+                         dlg._file_message_expander.get_label())
+        self.assertFalse(dlg._file_message_expander.get_expanded())
+        self.assertFalse(dlg._file_message_expander.get_property('sensitive'))
+
+    def test_file_selection_message(self):
+        """Selecting a file should bring up its commit message."""
+        tree = self.make_branch_and_tree('tree')
+        tree.branch.get_config().set_user_option('per_file_commits', 'true')
+        self.build_tree(['tree/a', 'tree/b/'])
+        tree.add(['a', 'b'], ['a-id', 'b-id'])
+
+        def get_file_text():
+            buf = dlg._file_message_text_view.get_buffer()
+            return buf.get_text(buf.get_start_iter(), buf.get_end_iter())
+
+        def get_saved_text(path):
+            """Get the saved text for a given record."""
+            return dlg._files_store.get_value(dlg._files_store.get_iter(path), 5)
+
+        dlg = commit.CommitDialog(tree)
+        self.assertEqual('File commit message',
+                         dlg._file_message_expander.get_label())
+        self.assertFalse(dlg._file_message_expander.get_expanded())
+        self.assertFalse(dlg._file_message_expander.get_property('sensitive'))
+        self.assertEqual('', get_file_text())
+
+        dlg._treeview_files.set_cursor((1,))
+        self.assertEqual('Commit message for a',
+                         dlg._file_message_expander.get_label())
+        self.assertTrue(dlg._file_message_expander.get_expanded())
+        self.assertTrue(dlg._file_message_expander.get_property('sensitive'))
+        self.assertEqual('', get_file_text())
+
+        self.assertEqual('', get_saved_text(1))
+        dlg._set_file_commit_message('Some text\nfor a\n')
+        dlg._save_current_file_message()
+        # We should have updated the ListStore with the new file commit info
+        self.assertEqual('Some text\nfor a\n', get_saved_text(1))
+
+        dlg._treeview_files.set_cursor((2,))
+        self.assertEqual('Commit message for b/',
+                         dlg._file_message_expander.get_label())
+        self.assertTrue(dlg._file_message_expander.get_expanded())
+        self.assertTrue(dlg._file_message_expander.get_property('sensitive'))
+        self.assertEqual('', get_file_text())
+
+        self.assertEqual('', get_saved_text(2))
+        dlg._set_file_commit_message('More text\nfor b\n')
+        # Now switch back to 'a'. The message should be saved, and the buffer
+        # should be updated with the other text
+        dlg._treeview_files.set_cursor((1,))
+        self.assertEqual('More text\nfor b\n', get_saved_text(2))
+        self.assertEqual('Commit message for a',
+                         dlg._file_message_expander.get_label())
+        self.assertTrue(dlg._file_message_expander.get_expanded())
+        self.assertTrue(dlg._file_message_expander.get_property('sensitive'))
+        self.assertEqual('Some text\nfor a\n', get_file_text())
+
+    def test_toggle_all_files(self):
+        """When checking the All Files entry, it should toggle all fields"""
+        tree = self.make_branch_and_tree('tree')
+        self.build_tree(['tree/a', 'tree/b/'])
+        tree.add(['a', 'b'], ['a-id', 'b-id'])
+
+        dlg = commit.CommitDialog(tree)
+        self.assertEqual([(None, None, True),
+                          ('a-id', 'a', True),
+                          ('b-id', 'b', True),
+                         ], [(r[0], r[1], r[2]) for r in dlg._files_store])
+
+        # TODO: jam 20071002 I'm not sure how to exactly trigger a toggle, it
+        #       looks like we need to call renderer.activate() and pass an
+        #       event and widget, and lots of other stuff I'm not sure what to
+        #       do with. So instead, we just call toggle directly, and assume
+        #       that toggle is hooked in correctly
+        # column = dlg._treeview_files.get_column(0)
+        # renderer = column.get_cell_renderers()[0]
+
+        # Toggle a single entry should set just that entry to False
+        dlg._toggle_commit(None, 1, dlg._files_store)
+        self.assertEqual([(None, None, True),
+                          ('a-id', 'a', False),
+                          ('b-id', 'b', True),
+                         ], [(r[0], r[1], r[2]) for r in dlg._files_store])
+
+        # Toggling the main entry should set all entries
+        dlg._toggle_commit(None, 0, dlg._files_store)
+        self.assertEqual([(None, None, False),
+                          ('a-id', 'a', False),
+                          ('b-id', 'b', False),
+                         ], [(r[0], r[1], r[2]) for r in dlg._files_store])
+
+        dlg._toggle_commit(None, 2, dlg._files_store)
+        self.assertEqual([(None, None, False),
+                          ('a-id', 'a', False),
+                          ('b-id', 'b', True),
+                         ], [(r[0], r[1], r[2]) for r in dlg._files_store])
+
+        dlg._toggle_commit(None, 0, dlg._files_store)
+        self.assertEqual([(None, None, True),
+                          ('a-id', 'a', True),
+                          ('b-id', 'b', True),
+                         ], [(r[0], r[1], r[2]) for r in dlg._files_store])
+
+    def test_specific_files(self):
+        tree = self.make_branch_and_tree('tree')
+        self.build_tree(['tree/a', 'tree/b/'])
+        tree.add(['a', 'b'], ['a-id', 'b-id'])
+
+        dlg = commit.CommitDialog(tree)
+        self.assertEqual((['a', 'b'], []), dlg._get_specific_files())
+
+        dlg._commit_selected_radio.set_active(True)
+        dlg._toggle_commit(None, 0, dlg._files_store)
+        self.assertEqual(([], []), dlg._get_specific_files())
+
+        dlg._toggle_commit(None, 1, dlg._files_store)
+        self.assertEqual((['a'], []), dlg._get_specific_files())
+
+    def test_specific_files_with_messages(self):
+        tree = self.make_branch_and_tree('tree')
+        tree.branch.get_config().set_user_option('per_file_commits', 'true')
+        self.build_tree(['tree/a_file', 'tree/b_dir/'])
+        tree.add(['a_file', 'b_dir'], ['1a-id', '0b-id'])
+
+        dlg = commit.CommitDialog(tree)
+        dlg._commit_selected_radio.set_active(True)
+        self.assertEqual((['a_file', 'b_dir'], []), dlg._get_specific_files())
+
+        dlg._treeview_files.set_cursor((1,))
+        dlg._set_file_commit_message('Test\nmessage\nfor a_file\n')
+        dlg._treeview_files.set_cursor((2,))
+        dlg._set_file_commit_message('message\nfor b_dir\n')
+
+        self.assertEqual((['a_file', 'b_dir'],
+                          [{'path':'a_file', 'file_id':'1a-id',
+                            'message':'Test\nmessage\nfor a_file\n'},
+                           {'path':'b_dir', 'file_id':'0b-id',
+                            'message':'message\nfor b_dir\n'},
+                          ]), dlg._get_specific_files())
+
+        dlg._toggle_commit(None, 1, dlg._files_store)
+        self.assertEqual((['b_dir'],
+                          [{'path':'b_dir', 'file_id':'0b-id',
+                            'message':'message\nfor b_dir\n'},
+                          ]), dlg._get_specific_files())
+
+
+class TestCommitDialog_Commit(tests.TestCaseWithTransport):
+    """Tests on the actual 'commit' button being pushed."""
+
+    def _set_question_yes(self, dlg):
+        """Set the dialog to answer YES to any questions."""
+        self.questions = []
+        def _question_yes(*args):
+            self.questions.append(args)
+            self.questions.append('YES')
+            return gtk.RESPONSE_YES
+        dlg._question_dialog = _question_yes
+
+    def _set_question_no(self, dlg):
+        """Set the dialog to answer NO to any questions."""
+        self.questions = []
+        def _question_no(*args):
+            self.questions.append(args)
+            self.questions.append('NO')
+            return gtk.RESPONSE_NO
+        dlg._question_dialog = _question_no
+
+    def test_bound_commit_local(self):
+        tree = self.make_branch_and_tree('tree')
+        self.build_tree(['tree/a'])
+        tree.add(['a'], ['a-id'])
+        rev_id1 = tree.commit('one')
+
+        tree2 = tree.bzrdir.sprout('tree2').open_workingtree()
+        self.build_tree(['tree2/b'])
+        tree2.add(['b'], ['b-id'])
+        tree2.branch.bind(tree.branch)
+
+        dlg = commit.CommitDialog(tree2)
+        # With the check box set, it should only effect the local branch
+        dlg._check_local.set_active(True)
+        dlg._set_global_commit_message('Commit message\n')
+        dlg._do_commit()
+
+        last_rev = tree2.last_revision()
+        self.assertEqual(last_rev, dlg.committed_revision_id)
+        self.assertEqual(rev_id1, tree.branch.last_revision())
+
+    def test_bound_commit_both(self):
+        tree = self.make_branch_and_tree('tree')
+        self.build_tree(['tree/a'])
+        tree.add(['a'], ['a-id'])
+        rev_id1 = tree.commit('one')
+
+        tree2 = tree.bzrdir.sprout('tree2').open_workingtree()
+        self.build_tree(['tree2/b'])
+        tree2.add(['b'], ['b-id'])
+        tree2.branch.bind(tree.branch)
+
+        dlg = commit.CommitDialog(tree2)
+        # With the check box set, it should only effect the local branch
+        dlg._check_local.set_active(False)
+        dlg._set_global_commit_message('Commit message\n')
+        dlg._do_commit()
+
+        last_rev = tree2.last_revision()
+        self.assertEqual(last_rev, dlg.committed_revision_id)
+        self.assertEqual(last_rev, tree.branch.last_revision())
+
+    def test_commit_no_message(self):
+        tree = self.make_branch_and_tree('tree')
+        self.build_tree(['tree/a', 'tree/b'])
+        tree.add(['a'], ['a-id'])
+        rev_id = tree.commit('one')
+
+        tree.add(['b'], ['b-id'])
+
+        dlg = commit.CommitDialog(tree)
+        self._set_question_no(dlg)
+        dlg._do_commit()
+        self.assertEqual(
+            [('Commit with an empty message?',
+              'You can describe your commit intent in the message.'),
+              'NO',
+            ], self.questions)
+        # By saying NO, nothing should be committed.
+        self.assertEqual(rev_id, tree.last_revision())
+        self.assertIs(None, dlg.committed_revision_id)
+        self.assertTrue(dlg._global_message_text_view.get_property('is-focus'))
+
+        self._set_question_yes(dlg)
+
+        dlg._do_commit()
+        self.assertEqual(
+            [('Commit with an empty message?',
+              'You can describe your commit intent in the message.'),
+              'YES',
+            ], self.questions)
+        committed = tree.last_revision()
+        self.assertNotEqual(rev_id, committed)
+        self.assertEqual(committed, dlg.committed_revision_id)
+
+    def test_initial_commit(self):
+        tree = self.make_branch_and_tree('tree')
+        self.build_tree(['tree/a'])
+        tree.add(['a'], ['a-id'])
+
+        dlg = commit.CommitDialog(tree)
+        dlg._set_global_commit_message('Some text\n')
+        dlg._do_commit()
+
+        last_rev = tree.last_revision()
+        self.assertEqual(last_rev, dlg.committed_revision_id)
+        rev = tree.branch.repository.get_revision(last_rev)
+        self.assertEqual(last_rev, rev.revision_id)
+        self.assertEqual('Some text\n', rev.message)
+
+    def test_pointless_commit(self):
+        tree = self.make_branch_and_tree('tree')
+        self.build_tree(['tree/a'])
+        tree.add(['a'], ['a-id'])
+        rev_id1 = tree.commit('one')
+
+        dlg = commit.CommitDialog(tree)
+        dlg._set_global_commit_message('Some text\n')
+
+        self._set_question_no(dlg)
+        dlg._do_commit()
+
+        self.assertIs(None, dlg.committed_revision_id)
+        self.assertEqual(rev_id1, tree.last_revision())
+        self.assertEqual(
+            [('Commit with no changes?',
+              'There are no changes in the working tree.'
+              ' Do you want to commit anyway?'),
+              'NO',
+            ], self.questions)
+
+        self._set_question_yes(dlg)
+        dlg._do_commit()
+
+        rev_id2 = tree.last_revision()
+        self.assertEqual(rev_id2, dlg.committed_revision_id)
+        self.assertNotEqual(rev_id1, rev_id2)
+        self.assertEqual(
+            [('Commit with no changes?',
+              'There are no changes in the working tree.'
+              ' Do you want to commit anyway?'),
+              'YES',
+            ], self.questions)
+
+    def test_unknowns(self):
+        """We should check if there are unknown files."""
+        tree = self.make_branch_and_tree('tree')
+        rev_id1 = tree.commit('one')
+        self.build_tree(['tree/a', 'tree/b'])
+        tree.add(['a'], ['a-id'])
+
+        dlg = commit.CommitDialog(tree)
+        dlg._set_global_commit_message('Some text\n')
+        self._set_question_no(dlg)
+
+        dlg._do_commit()
+
+        self.assertIs(None, dlg.committed_revision_id)
+        self.assertEqual(rev_id1, tree.last_revision())
+        self.assertEqual(
+            [("Commit with unknowns?",
+              "Unknown files exist in the working tree. Commit anyway?"),
+              "NO",
+            ], self.questions)
+
+        self._set_question_yes(dlg)
+        dlg._do_commit()
+
+        rev_id2 = tree.last_revision()
+        self.assertNotEqual(rev_id1, rev_id2)
+        self.assertEqual(rev_id2, dlg.committed_revision_id)
+        self.assertEqual(
+            [("Commit with unknowns?",
+              "Unknown files exist in the working tree. Commit anyway?"),
+              "YES",
+            ], self.questions)
+
+    def test_commit_specific_files(self):
+        tree = self.make_branch_and_tree('tree')
+        rev_id1 = tree.commit('one')
+        self.build_tree(['tree/a', 'tree/b'])
+        tree.add(['a', 'b'], ['a-id', 'b-id'])
+
+        dlg = commit.CommitDialog(tree)
+        dlg._commit_selected_radio.set_active(True) # enable partial
+        dlg._toggle_commit(None, 2, dlg._files_store) # unset 'b'
+
+        dlg._set_global_commit_message('Committing just "a"\n')
+        dlg._do_commit()
+
+        rev_id2 = dlg.committed_revision_id
+        self.assertIsNot(None, rev_id2)
+        self.assertEqual(rev_id2, tree.last_revision())
+
+        rt = tree.branch.repository.revision_tree(rev_id2)
+        entries = [(path, ie.file_id) for path, ie in rt.iter_entries_by_dir()
+                                       if path] # Ignore the root entry
+        self.assertEqual([('a', 'a-id')], entries)
+
+    def test_commit_partial_no_partial(self):
+        """Ignore the checkboxes if committing all files."""
+        tree = self.make_branch_and_tree('tree')
+        rev_id1 = tree.commit('one')
+        self.build_tree(['tree/a', 'tree/b'])
+        tree.add(['a', 'b'], ['a-id', 'b-id'])
+
+        dlg = commit.CommitDialog(tree)
+        dlg._commit_selected_radio.set_active(True) # enable partial
+        dlg._toggle_commit(None, 2, dlg._files_store) # unset 'b'
+
+        # Switch back to committing all changes
+        dlg._commit_all_files_radio.set_active(True)
+
+        dlg._set_global_commit_message('Committing everything\n')
+        dlg._do_commit()
+
+        rev_id2 = dlg.committed_revision_id
+        self.assertIsNot(None, rev_id2)
+        self.assertEqual(rev_id2, tree.last_revision())
+
+        rt = tree.branch.repository.revision_tree(rev_id2)
+        entries = [(path, ie.file_id) for path, ie in rt.iter_entries_by_dir()
+                                       if path] # Ignore the root entry
+        self.assertEqual([('a', 'a-id'), ('b', 'b-id')], entries)
+
+    def test_commit_no_messages(self):
+        tree = self.make_branch_and_tree('tree')
+        rev_id1 = tree.commit('one')
+        self.build_tree(['tree/a', 'tree/b'])
+        tree.add(['a', 'b'], ['a-id', 'b-id'])
+
+        dlg = commit.CommitDialog(tree)
+        dlg._set_global_commit_message('Simple commit\n')
+        dlg._do_commit()
+
+        rev = tree.branch.repository.get_revision(dlg.committed_revision_id)
+        self.failIf('file-info' in rev.properties)
+
+    def test_commit_disabled_messages(self):
+        tree = self.make_branch_and_tree('tree')
+        rev_id1 = tree.commit('one')
+
+        self.build_tree(['tree/a', 'tree/b'])
+        tree.add(['a', 'b'], ['a-id', 'b-id'])
+
+        dlg = commit.CommitDialog(tree)
+        self.assertFalse(dlg._file_message_expander.get_property('visible'))
+        self.assertEqual('Commit Message',
+                         dlg._global_message_label.get_text())
+
+        tree.branch.get_config().set_user_option('per_file_commits', 'true')
+        dlg = commit.CommitDialog(tree)
+        self.assertTrue(dlg._file_message_expander.get_property('visible'))
+        self.assertEqual('Global Commit Message',
+                         dlg._global_message_label.get_text())
+
+        tree.branch.get_config().set_user_option('per_file_commits', 'on')
+        dlg = commit.CommitDialog(tree)
+        self.assertTrue(dlg._file_message_expander.get_property('visible'))
+        self.assertEqual('Global Commit Message',
+                         dlg._global_message_label.get_text())
+
+        tree.branch.get_config().set_user_option('per_file_commits', 'y')
+        dlg = commit.CommitDialog(tree)
+        self.assertTrue(dlg._file_message_expander.get_property('visible'))
+        self.assertEqual('Global Commit Message',
+                         dlg._global_message_label.get_text())
+
+        tree.branch.get_config().set_user_option('per_file_commits', 'n')
+        dlg = commit.CommitDialog(tree)
+        self.assertFalse(dlg._file_message_expander.get_property('visible'))
+        self.assertEqual('Commit Message',
+                         dlg._global_message_label.get_text())
+
+    def test_commit_specific_files_with_messages(self):
+        tree = self.make_branch_and_tree('tree')
+        tree.branch.get_config().set_user_option('per_file_commits', 'true')
+        rev_id1 = tree.commit('one')
+        self.build_tree(['tree/a', 'tree/b'])
+        tree.add(['a', 'b'], ['a-id', 'b-id'])
+
+        dlg = commit.CommitDialog(tree)
+        dlg._commit_selected_radio.set_active(True) # enable partial
+        dlg._treeview_files.set_cursor((1,))
+        dlg._set_file_commit_message('Message for A\n')
+        dlg._treeview_files.set_cursor((2,))
+        dlg._set_file_commit_message('Message for B\n')
+        dlg._toggle_commit(None, 2, dlg._files_store) # unset 'b'
+        dlg._set_global_commit_message('Commit just "a"')
+
+        dlg._do_commit()
+
+        rev_id2 = dlg.committed_revision_id
+        self.assertEqual(rev_id2, tree.last_revision())
+        rev = tree.branch.repository.get_revision(rev_id2)
+        self.assertEqual('Commit just "a"', rev.message)
+        file_info = rev.properties['file-info']
+        self.assertEqual('ld7:file_id4:a-id'
+                           '7:message14:Message for A\n'
+                           '4:path1:a'
+                         'ee', file_info)
+        self.assertEqual([{'path':'a', 'file_id':'a-id',
+                           'message':'Message for A\n'},
+                         ], bencode.bdecode(file_info))
+
+    def test_commit_messages_after_merge(self):
+        tree = self.make_branch_and_tree('tree')
+        tree.branch.get_config().set_user_option('per_file_commits', 'true')
+        rev_id1 = tree.commit('one')
+        tree2 = tree.bzrdir.sprout('tree2').open_workingtree()
+        self.build_tree(['tree2/a', 'tree2/b'])
+        tree2.add(['a', 'b'], ['a-id', 'b-id'])
+        rev_id2 = tree2.commit('two')
+
+        tree.merge_from_branch(tree2.branch)
+
+        dlg = commit.CommitDialog(tree)
+        dlg._treeview_files.set_cursor((1,)) # 'a'
+        dlg._set_file_commit_message('Message for A\n')
+        # No message for 'B'
+        dlg._set_global_commit_message('Merging from "tree2"\n')
+
+        dlg._do_commit()
+
+        rev_id3 = dlg.committed_revision_id
+        self.assertEqual(rev_id3, tree.last_revision())
+        rev = tree.branch.repository.get_revision(rev_id3)
+        self.assertEqual('Merging from "tree2"\n', rev.message)
+        self.assertEqual([rev_id1, rev_id2], rev.parent_ids)
+        file_info = rev.properties['file-info']
+        self.assertEqual('ld7:file_id4:a-id'
+                           '7:message14:Message for A\n'
+                           '4:path1:a'
+                         'ee', file_info)
+        self.assertEqual([{'path':'a', 'file_id':'a-id',
+                           'message':'Message for A\n'},
+                         ], bencode.bdecode(file_info))
+
+    def test_commit_unicode_messages(self):
+        from bzrlib.tests.test_diff import UnicodeFilename
+        self.requireFeature(UnicodeFilename)
+
+        tree = self.make_branch_and_tree('tree')
+        tree.branch.get_config().set_user_option('per_file_commits', 'true')
+        self.build_tree(['tree/a', u'tree/\u03a9'])
+        tree.add(['a', u'\u03a9'], ['a-id', 'omega-id'])
+
+        dlg = commit.CommitDialog(tree)
+        dlg._treeview_files.set_cursor((1,)) # 'a'
+        dlg._set_file_commit_message(u'Test \xfan\xecc\xf6de\n')
+        dlg._treeview_files.set_cursor((2,)) # omega
+        dlg._set_file_commit_message(u'\u03a9 is the end of all things.\n')
+        dlg._set_global_commit_message(u'\u03a9 and \xfan\xecc\xf6de\n')
+
+        self.assertEqual(([u'a', u'\u03a9'],
+                          [{'path':'a', 'file_id':'a-id',
+                            'message':'Test \xc3\xban\xc3\xacc\xc3\xb6de\n'},
+                           {'path':'\xce\xa9', 'file_id':'omega-id',
+                            'message':'\xce\xa9 is the end of all things.\n'},
+                          ]), dlg._get_specific_files())
+
+        dlg._do_commit()
+
+        rev = tree.branch.repository.get_revision(dlg.committed_revision_id)
+        file_info = rev.properties['file-info'].encode('UTF-8')
+        value = ('ld7:file_id4:a-id'
+                   '7:message16:Test \xc3\xban\xc3\xacc\xc3\xb6de\n'
+                   '4:path1:a'
+                  'e'
+                  'd7:file_id8:omega-id'
+                   '7:message29:\xce\xa9 is the end of all things.\n'
+                   '4:path2:\xce\xa9'
+                  'e'
+                 'e')
+        self.assertEqual(value, file_info)
+        file_info_decoded = bencode.bdecode(file_info)
+        for d in file_info_decoded:
+            d['path'] = d['path'].decode('UTF-8')
+            d['message'] = d['message'].decode('UTF-8')
+
+        self.assertEqual([{'path':u'a', 'file_id':'a-id',
+                           'message':u'Test \xfan\xecc\xf6de\n'},
+                          {'path':u'\u03a9', 'file_id':'omega-id',
+                           'message':u'\u03a9 is the end of all things.\n'},
+                         ], file_info_decoded)

=== modified file 'tests/test_diff.py'
--- tests/test_diff.py	2007-07-16 10:27:35 +0000
+++ tests/test_diff.py	2007-10-02 23:08:12 +0000
@@ -16,11 +16,15 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
 from cStringIO import StringIO
-
-from bzrlib.plugins.gtk.diff import DiffWindow
-from bzrlib.tests import TestCase
-
-class TestDiffWindow(TestCase):
+import os
+
+from bzrlib import tests
+
+from bzrlib.plugins.gtk.diff import DiffView, _iter_changes_to_status
+
+
+class TestDiffViewSimple(tests.TestCase):
+
     def test_parse_colordiffrc(self):
         colordiffrc = '''\
 newtext=blue
@@ -35,5 +39,129 @@
                 'oldtext': 'Red',
                 'diffstuff': '#ffff00',
         }
-        parsed_colors = DiffWindow.parse_colordiffrc(StringIO(colordiffrc))
+        parsed_colors = DiffView.parse_colordiffrc(StringIO(colordiffrc))
         self.assertEqual(colors, parsed_colors)
+
+
+class TestDiffView(tests.TestCaseWithTransport):
+
+    def test_unicode(self):
+        from bzrlib.tests.test_diff import UnicodeFilename
+        self.requireFeature(UnicodeFilename)
+
+        tree = self.make_branch_and_tree('tree')
+        self.build_tree([u'tree/\u03a9'])
+        tree.add([u'\u03a9'], ['omega-id'])
+
+        view = DiffView()
+        view.set_trees(tree, tree.basis_tree())
+        view.show_diff(None)
+        buf = view.buffer
+        start, end = buf.get_bounds()
+        text = buf.get_text(start, end)
+        self.assertContainsRe(text,
+            "=== added file '\xce\xa9'\n"
+            '--- .*\t1970-01-01 00:00:00 \\+0000\n'
+            r'\+\+\+ .*\t\d\d\d\d-\d\d-\d\d \d\d:\d\d:\d\d [+-]\d\d\d\d\n'
+            '@@ -0,0 \\+1,1 @@\n'
+            '\\+contents of tree/\xce\xa9\n'
+            '\n'
+            )
+
+
+class Test_IterChangesToStatus(tests.TestCaseWithTransport):
+
+    def assertStatusEqual(self, expected, tree):
+        values = _iter_changes_to_status(tree.basis_tree(), tree)
+        self.assertEqual(expected, values)
+
+    def test_status_added(self):
+        tree = self.make_branch_and_tree('tree')
+        self.build_tree(['tree/a', 'tree/b/', 'tree/b/c'])
+        tree.add(['a', 'b', 'b/c'], ['a-id', 'b-id', 'c-id'])
+
+        self.assertStatusEqual(
+            [('a-id', 'a', 'added', 'a'),
+             ('b-id', 'b', 'added', 'b/'),
+             ('c-id', 'b/c', 'added', 'b/c'),
+            ], tree)
+
+    def test_status_renamed(self):
+        tree = self.make_branch_and_tree('tree')
+        self.build_tree(['tree/a', 'tree/b/', 'tree/b/c'])
+        tree.add(['a', 'b', 'b/c'], ['a-id', 'b-id', 'c-id'])
+        rev_id1 = tree.commit('one')
+
+        tree.rename_one('b', 'd')
+        tree.rename_one('a', 'd/a')
+
+        self.assertStatusEqual(
+            [('b-id', 'd', 'renamed', 'b/ => d/'),
+             ('a-id', 'd/a', 'renamed', 'a => d/a'),
+            ], tree)
+
+    def test_status_modified(self):
+        tree = self.make_branch_and_tree('tree')
+        self.build_tree(['tree/a'])
+        tree.add(['a'], ['a-id'])
+        rev_id1 = tree.commit('one')
+
+        self.build_tree_contents([('tree/a', 'new contents for a\n')])
+
+        self.assertStatusEqual(
+            [('a-id', 'a', 'modified', 'a'),
+            ], tree)
+
+    def test_status_renamed_and_modified(self):
+        tree = self.make_branch_and_tree('tree')
+        self.build_tree(['tree/a', 'tree/b/', 'tree/b/c'])
+        tree.add(['a', 'b', 'b/c'], ['a-id', 'b-id', 'c-id'])
+        rev_id1 = tree.commit('one')
+
+        tree.rename_one('b', 'd')
+        tree.rename_one('a', 'd/a')
+        self.build_tree_contents([('tree/d/a', 'new contents for a\n'),
+                                  ('tree/d/c', 'new contents for c\n'),
+                                 ])
+        # 'c' is not considered renamed, because only its parent was moved, it
+        # stayed in the same directory
+
+        self.assertStatusEqual(
+            [('b-id', 'd', 'renamed', 'b/ => d/'),
+             ('a-id', 'd/a', 'renamed and modified', 'a => d/a'),
+             ('c-id', 'd/c', 'modified', 'd/c'),
+            ], tree)
+
+    def test_status_kind_changed(self):
+        tree = self.make_branch_and_tree('tree')
+        self.build_tree(['tree/a', 'tree/b'])
+        tree.add(['a', 'b'], ['a-id', 'b-id'])
+        tree.commit('one')
+
+        os.remove('tree/a')
+        self.build_tree(['tree/a/'])
+        # XXX:  This is technically valid, and the file list handles it fine,
+        #       but 'show_diff_trees()' does not, so we skip this part of the
+        #       test for now.
+        # tree.rename_one('b', 'c')
+        # os.remove('tree/c')
+        # self.build_tree(['tree/c/'])
+
+        self.assertStatusEqual(
+            [('a-id', 'a', 'kind changed', 'a => a/'),
+            # ('b-id', 'c', True, 'b => c/', 'renamed and modified'),
+            ], tree)
+
+    def test_status_removed(self):
+        tree = self.make_branch_and_tree('tree')
+        self.build_tree(['tree/a', 'tree/b/'])
+        tree.add(['a', 'b'], ['a-id', 'b-id'])
+        tree.commit('one')
+
+        os.remove('tree/a')
+        tree.remove('b', force=True)
+
+        self.assertStatusEqual(
+            [('a-id', 'a', 'removed', 'a'),
+             ('b-id', 'b', 'removed', 'b/'),
+            ], tree)

=== modified file 'viz/branchwin.py'
--- viz/branchwin.py	2007-11-07 12:13:10 +0000
+++ viz/branchwin.py	2007-11-07 14:19:09 +0000
@@ -18,6 +18,7 @@
 from bzrlib.plugins.gtk.preferences import PreferencesWindow
 from bzrlib.revision import Revision
 from bzrlib.config import BranchConfig
+from bzrlib.config import GlobalConfig
 from treeview import TreeView
 
 class BranchWindow(Window):
@@ -39,9 +40,15 @@
         Window.__init__(self, parent=parent)
         self.set_border_width(0)
 
-        self.branch = branch
-        self.start  = start
-        self.maxnum = maxnum
+        self.branch      = branch
+        self.start       = start
+        self.maxnum      = maxnum
+        self.config      = GlobalConfig()
+
+        if self.config.get_user_option('viz-compact-view') == 'yes':
+            self.compact_view = True
+        else:
+            self.compact_view = False
 
         self.set_title(branch.nick + " - revision history")
 
@@ -61,6 +68,9 @@
 
         self.construct()
 
+    def set_revision(self, revision):
+        self.treeview.set_revision(revision)
+
     def construct(self):
         """Construct the window contents."""
         vbox = gtk.VBox(spacing=0)
@@ -192,7 +202,7 @@
         image_loading.show()
         
         label_loading = gtk.Label(_("Please wait, loading ancestral graph..."))
-        label_loading.set_alignment(0.0, 0.5)  
+        label_loading.set_alignment(0.0, 0.5)
         label_loading.show()
         
         self.loading_msg_box = gtk.HBox()
@@ -207,7 +217,12 @@
     def construct_top(self):
         """Construct the top-half of the window."""
         # FIXME: Make broken_line_length configurable
-        self.treeview = TreeView(self.branch, self.start, self.maxnum, 32)
+        if self.compact_view:
+            brokenlines = 32
+        else:
+            brokenlines = None
+
+        self.treeview = TreeView(self.branch, self.start, self.maxnum, brokenlines)
 
         self.treeview.connect("revision-selected",
                 self._treeselection_changed_cb)
@@ -229,16 +244,23 @@
                                          0, 0)
         self.back_button.connect("clicked", self._back_clicked_cb)
         self.toolbar.insert(self.back_button, -1)
-        self.back_button.show()
 
         self.fwd_button = gtk.MenuToolButton(stock_id=gtk.STOCK_GO_UP)
         self.fwd_button.add_accelerator("clicked", self.accel_group, ord(']'),
                                         0, 0)
         self.fwd_button.connect("clicked", self._fwd_clicked_cb)
         self.toolbar.insert(self.fwd_button, -1)
-        self.fwd_button.show()
-
-        self.toolbar.show()
+
+        self.toolbar.insert(gtk.SeparatorToolItem(), -1)
+
+        brokenlines_button = gtk.ToggleToolButton()
+        brokenlines_button.set_label("Compact View")
+        brokenlines_button.set_active(self.compact_view)
+        brokenlines_button.set_is_important(True)
+        brokenlines_button.connect('toggled', self._brokenlines_toggled_cb)
+        self.toolbar.insert(brokenlines_button, -1)
+
+        self.toolbar.show_all()
 
         return self.toolbar
 
@@ -330,6 +352,23 @@
     def _set_revision_cb(self, w, revision_id):
         self.treeview.set_revision_id(revision_id)
 
+    def _brokenlines_toggled_cb(self, button):
+        self.compact_view = button.get_active()
+
+        if self.compact_view:
+            option = 'yes'
+        else:
+            option = 'no'
+
+        self.config.set_user_option('viz-compact-view', option)
+
+        revision = self.treeview.get_revision()
+
+        self.treeview.destroy()
+        self.paned.pack1(self.construct_top(), resize=True, shrink=False)
+
+        gobject.idle_add(self.set_revision, revision.revision_id)
+
     def _tag_revision_cb(self, w):
         dialog = AddTagDialog(self.branch.repository, self.treeview.get_revision().revision_id, self.branch)
         response = dialog.run()

=== modified file 'viz/treeview.py'
--- viz/treeview.py	2007-11-06 18:02:09 +0000
+++ viz/treeview.py	2007-11-07 14:19:09 +0000
@@ -224,7 +224,11 @@
 
         self.treeview.set_rules_hint(True)
         self.treeview.set_search_column(treemodel.REVNO)
-        self.treeview.set_tooltip_column(treemodel.MESSAGE)
+        
+        # Fix old PyGTK bug - by JAM
+        set_tooltip = getattr(self.treeview, 'set_tooltip_column', None)
+        if set_tooltip is not None:
+            set_tooltip(treemodel.MESSAGE)
 
         self.treeview.get_selection().connect("changed",
                 self._on_selection_changed)

